/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/


import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;

import uk.ac.essex.malexa.nlp.dp.GuiTAR.DiscourseModelImplementer;
import libsvm.svm;
import libsvm.SVMScalingModel;
import java.io.FileNotFoundException;
import libsvm.svm_model;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.Local;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.IOXMLUtils;
import org.w3c.dom.Document;

/**
 * <b>Description:</b><br>
 * This class is the old main running class for the GuiTAR tool. It should be called
 * from a command with the right parameters.<br>
 * @author Mijail A. Kabadjov
 * @author Philippe J. Goux
 * @version 1.0
 */
public class GTAR_Runner {
    public static int process(String file, int queryCounter, svm_model svmModel, SVMScalingModel svmScalingModel) throws IOException {
        Local.out.println("File: " + file);
        Document domDoc = IOXMLUtils.load(file); //Load XML file

        if (domDoc == null)
            throw new IOException("Error: Could not load file " + file);

        DiscourseModelImplementer dmi = new DiscourseModelImplementer(domDoc, queryCounter, svmModel, svmScalingModel);
        dmi.process();
        IOXMLUtils.save(dmi.getDOMDocument(), "processed." + file); //Store modified DOM Tree to file
        return dmi.globalQueryCounter;
    }


    //Main method for testing
    public static void main(String args[]) throws IOException {
        // Load gnmvpc.libsvm.* files
        svm_model svmModel = null;
        SVMScalingModel svmScalingModel = null;
        try {
            svmModel = svm.svm_load_model("gnmvpc.libsvm" + ".model"); //modelFileName = "gnmvpc.libsvm"
            svmScalingModel = new SVMScalingModel("gnmvpc.libsvm" + ".range");
        } catch (FileNotFoundException fnfe) {
            Local.err.printException(fnfe);
            System.exit(1);
        } catch (IOException ioe) {
            Local.err.printException(ioe);
            System.exit(1);
        }

        // Look for arguments
        if (args.length < 1) {
            BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
            String fileName = in.readLine();
            int qCounter = 0;
            while (fileName != null) {
                qCounter = process(fileName, qCounter, svmModel, svmScalingModel);
                fileName = in.readLine();
            }
        } else {
            process(args[0], 0, svmModel, svmScalingModel);
        } //end if-else
    }

}

