/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp;

import java.util.Set;
import java.util.Iterator;

/**
 * A class that provides static methods for computing precision, recall, F, E, Fallout, and accuracy.
 */
public class NLPEvaluationMeasures {


	public static double precision( Set selected, Set target ) {
		double result = 0;
		double tp = 0, fp = 0; //true and false positives
		Iterator it = target.iterator(); //get an iterator
		while( it.hasNext() ) {
			if( selected.contains( it.next() ) ) tp++;
		}
		fp = selected.size() - tp;
		result = 100 * tp / ( tp + fp ); //though tp+fp = selected.size()
		return result;
	}

	public static double recall( Set selected, Set target ) {
		double result = 0;
		double tp = 0, fn = 0; //true positives and false negatives
		Iterator it = target.iterator(); //get an iterator
		while( it.hasNext() ) {
			if( selected.contains( it.next() ) ) tp++;
		}
		fn = target.size() - tp;
		result = 100 * tp / ( tp + fn ); //though tp+fn = target.size()
		return result;
	}

	public static double fMeasure( Set selected, Set target, double alpha ) {
		double result = 0;
		double oneOverP = 1 / NLPEvaluationMeasures.precision( selected, target );
		double oneOverR = 1 / NLPEvaluationMeasures.recall( selected, target );
		result = 1 / ( alpha*oneOverP + (1 - alpha)*oneOverR );
		return result;
	}

	//Default F measure
	public static double fMeasure( Set selected, Set target ) {
		return NLPEvaluationMeasures.fMeasure( selected, target, 0.5 );
	}

	public static double eMeasure( Set selected, Set target, double alpha ) {
		return ( 1 - NLPEvaluationMeasures.fMeasure( selected, target, alpha ) );
	}

	//Default E measure
	public static double eMeasure( Set selected, Set target ) {
		return NLPEvaluationMeasures.eMeasure( selected, target, 0.5 );
	}

	public static double fallout( Set selected, Set target, Set universe ) {
		double result = 0;
		double tp = 0, fp = 0, tn = 0; //true-, false positives, and true negatives
		Iterator it = target.iterator(); //get an iterator
		while( it.hasNext() ) {
			if( selected.contains( it.next() ) ) tp++;
		}
		fp = selected.size() - tp;
		tn = universe.size() - target.size() - fp;
		result = 100 * fp / ( fp + tn );
		return result;
	}

	public static double accuracy( Set selected, Set target, Set universe ) {
		double result = 0;
		double tp = 0, fp = 0, tn = 0; //true-, false positives, and true negatives
		Iterator it = target.iterator(); //get an iterator
		while( it.hasNext() ) {
			if( selected.contains( it.next() ) ) tp++;
		}
		fp = selected.size() - tp;
		tn = universe.size() - target.size() - fp;
		long total = universe.size();
		result = 100 * ( tp + tn ) / ( total );
		return result;
	}

} //end class NLPEvaluationMeasures
