/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR;

import java.util.Vector;
import org.w3c.dom.Node;

/**
 * A class that encapsulates the general functionality of a Segment and provides implementation
 * to the methods defined by the interface DiscourseSegmentation.
 * @author Mijail A. Kabadjov
 * @version 1.1
 */
public interface DiscourseSegmentation {

	/**
	 * Retrieves the DOM node of this segment.
	 * @return Node The node
	 */
	public Node getNode();

	/**
	 * Retrieves the relationship holding between this segment and its parent.
	 * @return Object The relation with parent segment
	 */
	public Object getRelationWithParent();

	/**
	 * Retrieves the parent segment of this segment.
	 * @return Segment The parent segment
	 */
	public Segment getParent();

	/**
	 * Retrieves the previous sibling of this segment, which could be another Segment or an Utterance.
	 * @return Segment The left sibling
	 */
	public Segment getPrevSibling();

	/**
	 * Retrieves the next sibling of this segment, which could be another Segment or an Utterance.
	 * @return Segment The right sibling
	 */
	public Segment getNextSibling();

	/**
	 * Retrieves the list of children of this segment.
	 * Returns a polimoriphic Vector containing objects of class Segment or Utterance.
	 * @return Vector The list of Segments and/or Utterances
	 */
	public Vector getChildren();

	/**
	 * Adds a new child to the segment and updates children pointers to prev-/nextSibling.
	 * @param child The child (Utterance/Segment) to be added
	 */
	public void addChild( Segment child );

	/**
	 * Sets the previous sibling of this segment.
	 * @param sibling The sibling
	 */
	public void setPreviousSibling( Segment sibling );

	/**
	 * Sets the next sibling of this segment.
	 * @param sibling The sibling
	 */
	public void setNextSibling( Segment sibling );


} //end interface DiscourseSegmentation
