/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/


package uk.ac.essex.malexa.nlp.dp.GuiTAR;

import uk.ac.essex.malexa.nlp.dp.GuiTAR.la.*;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.Local;
import java.io.File;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.IniFile;

//import uk.ac.essex.malexa.ARE.Agreement;
//import stemmer.Stemmer;

/**
 * A class that encapsulates the lexical information of a word.
 * @author Mijail A. Kabadjov
 * @author Philippe J. Goux
 */
public class LexicalWord implements Sense {

    //Constants
    private static final String STEMMING_RULES_FILE = "stemmer/rules";
    private static final String PART_OF_SPEECH = "pos";

    //Class variables
    private String word; //Stores the word itself
    private String stem; //Stores the stem of the word
    private String lemma; //Stores the lemma of the word
    private String posTag; //Part-of-speech (POS) tag
    private Agreement agreement; //Stores the agreement features (NP/VP head)

    //Constructor
    public LexicalWord(String theWord, String thePOSTag) {
        word = theWord;
        posTag = thePOSTag;
        agreement = Agreement.newInstance(thePOSTag, theWord);
    }


    //Accessors
    public String getWord() {
        return word;
    }


    public String getStem() {
        return stem;
    }


    public String getPOSTag() {
        return posTag;
    }


    public Agreement getAgreement() {
        return agreement;
    }


    //Override toString method
    public String toString() {
        return (getWord());
        /****	return(
          "\n\t\t" + "Word: " + getWord() +
          "\n\t\t" + "Stem: " + getStem() +
          "\n\t\t" + "POS-Tag: " + getPOSTag() +
          "\n\t\t" + "Agreement: " + getAgreement()
          );
         ***/
    }


    /*************** Sense methods ****************/
    /***** Very simple implementations for the moment *********/

    /**
     * Returns the original word.
     * @return String The word entry
     */
    public String getEntry() {
        return (getWord());
    }


    /**
     * Returns the sense of the word.
     * @return Object The sense
     */
    public Object getSense() {
        return (getWord());
    }


    /**
     * Returns the similarity between Sense s and this sense.
     * Depending on the implementation might range from [0 - 1] or [-1 - 1].
     * @param s A sense to be compared with this sense
     * @return double The similarity between s and this sense
     */
    public double senseSimilarity(Sense s) {
        return (word.compareTo((String) s.getSense()));
    }


    /**
     * Tests for equality between Object o and this object.
     * @param o The object to be compared with this object
     * @return boolean True if the test succeeds, false otherwise
     */
    public boolean equals(Object o) {
        return (getSense().equals(((Sense) o).getSense()));
    }


    /* ** It seems that this method has to be overriden as well, in order for the the set framework to work ** */
    public int hashCode() {
        return (((String) getSense()).hashCode());
    }
} //End class LexicalWord


