/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/


package uk.ac.essex.malexa.nlp.dp.GuiTAR;

//For DOM Tree manipulation
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

//Others
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.Local;
import java.util.Vector;
import java.util.Map;
import java.util.HashMap;
//IO
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;

/**
 * A class that passes a reference annotation over to an automatically parsed and processed file.
 * In the process NE alignment takes place, so the reference annotation is expressed in terms of
 * the parsed markables neIds. It assumes the reference annotation comes from a file in MMAX2 format.
 */
public class NeAlignerGNM {


     Map<String, String> mapNeIdToMinWords;
     Map<String, Vector<String>> mapCorpusIdToParsedId, mapParsedIdToCorpusId;
     Map<String, String> mapCorpusIdToCorpusPhrase, mapParsedIdToParsedPhrase;

     public NeAlignerGNM() {
	mapNeIdToMinWords = new HashMap<String, String>();
	mapCorpusIdToParsedId = new HashMap<String, Vector<String>>();
	mapParsedIdToCorpusId = new HashMap<String, Vector<String>>();
	mapCorpusIdToCorpusPhrase = new HashMap<String, String>();
	mapParsedIdToParsedPhrase = new HashMap<String, String>();
     }

     /**
      * This is the main method of the NeAligner. It reads information from three sources: 1. ..._coref_level.xml (i.e., MMAX2 file);
      * 2. ..._mas.xmluntagged.xml (i.e., mas file generated from MMAX2); 3. processed...xml (i.e., file processed by GuiTAR and before that,
      * by Charniak's parser and Olivia's converter). The result is a translation of the annotation from hand-annotated markable ids to 
      * automatically parsed markable ids.
      * @param inputFileName The name of the file to be processed (e.g. A0222)
      */
     public void processFile( String inputFileName ) {
	//Load the three files
	//the commented are for MUC
	//Document docParsed = IOXMLUtils.load( "processed.masxml.tagged." + inputFileName + "_mas.xmluntagged.xml.txt.xml" ); //Load XML file
Document docParsed = IOXMLUtils.load( "processed.masxml." + inputFileName + ".parsed.xml" ); //Load XML file
	//Document docReference = IOXMLUtils.load( inputFileName + "_mas.xmluntagged.xml" ); //Load XML file
Document docReference = IOXMLUtils.load( "masxml." + inputFileName + "_tagged.xml" ); //Load XML file
	//Document docCorefLevel = IOXMLUtils.load( inputFileName + "_coref_level.xml" ); //Load XML file
Document docCorefLevel = null; //comment this for MUC, uncomment the above line
	//NOT SURE HOW TO USE THE min_words - see markable_26 in A9569
//uncomment this for MUC
//buildMapNeIdToMinWords( docCorefLevel ); //This map is used only within method matchNEs()
	buildMapCorpusIdToParsedId( docReference, docParsed );
	NodeList anteEvaluated = docReference.getElementsByTagName( "ante" );
	/*** Import ante elements from evaluated to processed ***/
	NodeList secNodes = docParsed.getElementsByTagName( "section" );
	Node sectionNode = secNodes.item( 0 );
	for( int i=0; i < anteEvaluated.getLength(); i++ ) {
		//Only for VPC and GNOME; comment next line when processing MUC
		if(!identityRelation( (Element)anteEvaluated.item( i ) )) continue; //Skip non-identity relations
		Node newNode = docParsed.importNode( anteEvaluated.item( i ), true );
		/** Change the corpusId to parsedId **/
		changeAttributeId( (Element)newNode, "current" );
		changeAttributeId( (Element)(newNode.getFirstChild()), "antecedent" );
		sectionNode.appendChild( newNode );
	}
	//Save the resulting file
	IOXMLUtils.save( docParsed, "processed.masxml." + inputFileName + ".parsed.aligned.xml" ); //Store modified DOM Tree to file
     }

     /**
      * A method that checks whether the annotated relation is identity.
      * param ante The ante node as Element
      * return boolean True if identity link, false otherwise
      */
     private boolean identityRelation( Element ante ) {
	return ( ante.getAttribute( "rel" ).equals( "ident" ) );
     }

     /**
      * A method that translates the id of a corpus NE to an id of a matching parsed NE.
      * @param element This is the imported <ante> element
      * @param attrName This is either "current" or "antecedent"
      */
     private void changeAttributeId( Element element, String attrName ) {
	String attr = element.getAttribute( attrName );
	System.out.println("Corpus^" + attr + "^ ^" + mapCorpusIdToCorpusPhrase.get(attr) + "^" + attrName + "^" + mapNeIdToMinWords.get(attr));
	//HERE TRANSLATE THE ID USING THE MAPCORPUSIDTOPARSEDID AND PRINT ANY RELEVANT INFORMATION
	Vector<String> vecIds = mapCorpusIdToParsedId.get( attr );
	String newAttr = null;
	if( vecIds == null )
		System.out.println(">>^No match!");
	else {
		newAttr = vecIds.elementAt( 0 );
		//Print all the options
		for( int i = 0; i < vecIds.size(); i++) 
			System.out.println(">>Parsed^" + vecIds.elementAt( i ) + "^" + printVector( mapParsedIdToCorpusId.get( vecIds.elementAt( i ) ) ) + "^" + mapParsedIdToParsedPhrase.get(vecIds.elementAt( i )));
	}
	element.setAttribute( attrName, newAttr );
      }

     /**
      * Prints the elements of a Vector separated by commas.
      * @param vecIds The Vector of ne ids
      * @return String The ids separated by commas
      */
     private String printVector( Vector<String> vecIds ) {
	StringBuffer result = new StringBuffer();
	if( vecIds != null ) for (int i = 0; i < vecIds.size(); i++) result.append( vecIds.elementAt( i ) + ",");
	return ( result.toString() );
     }

     /**
      * Builds a map of MMAX2 markable ids to min_words attribute values.
      * @param doc The DOM document generated from ..._coref_level.xml file
      */
     private void buildMapNeIdToMinWords( Document doc ) {
	NodeList markables = doc.getElementsByTagName( "markable" );
	for( int i = 0; i < markables.getLength(); i++ ) {
		String id = ((Element)markables.item( i )).getAttribute( "id" );
		String minWords = ((Element)markables.item( i )).getAttribute( "min_words" );
		mapNeIdToMinWords.put( id, minWords );
	}
     }

     /**
      * NEs are matched sentence by sentence, exploiting the nice characteristic that sentences in the corpus and parsed files
      * are aligned.
      * @param docReference The DOM document generated from ..._mas.xmluntagged.xml file
      * @param docParsed The DOM document generated from processed...xml file
      */
     private void buildMapCorpusIdToParsedId( Document docReference, Document docParsed ) {
	//NodeList corpusSentences = docReference.getElementsByTagName( "s" );
	Vector<Node> vCorpusSentences = getSentences( docReference );
	//NodeList parsedSentences = docParsed.getElementsByTagName( "s" );
	Vector<Node> vParsedSentences = getSentences( docParsed );
	for( int i = 0; i < vCorpusSentences.size(); i++ ) {
		Element corpusSent = (Element)vCorpusSentences.elementAt( i );
		Element parsedSent = (Element)vParsedSentences.elementAt( i );
		NodeList corpusNEs = corpusSent.getElementsByTagName( "ne" );
		NodeList parsedNEs = parsedSent.getElementsByTagName( "ne" );
		Map<String, NEPosition> corpusNeIdToPosition = new HashMap<String, NEPosition>();
		Map<String, NEPosition> parsedNeIdToPosition = new HashMap<String, NEPosition>();

		corpusNeIdToPosition.put();

		//int idxLastParsedNEMatched = 0;
		for( int j = 0; j < corpusNEs.getLength(); j++ ) {
			Element corpusNE = (Element)corpusNEs.item( j );
			mapCorpusIdToCorpusPhrase.put( corpusNE.getAttribute("id"), getNePhrase( corpusNE ) );
			int maxMatch = 0, auxMatch = 0;
			String maxMatchCorpusNEId = null, maxMatchParsedNEId = null;
			//for( int k = (idxLastParsedNEMatched < 2 ? 0 : idxLastParsedNEMatched - 2); k < (k + 2) && k < parsedNEs.getLength(); k++ ) { //Look only 2 NEs ahead and back
			for( int k = 0; k < parsedNEs.getLength(); k++ ) {
				Element parsedNE = (Element)parsedNEs.item( k );
				mapParsedIdToParsedPhrase.put( parsedNE.getAttribute("id"), getNePhrase( parsedNE ) );
				if( (auxMatch = matchNEs( corpusNE, parsedNE )) > maxMatch ) {
					maxMatch = auxMatch;
					maxMatchCorpusNEId = corpusNE.getAttribute("id");
					maxMatchParsedNEId = parsedNE.getAttribute("id");
					//idxLastParsedNEMatched = k;
				}
			} //end inner for - k
			if( maxMatch > 0 ) {
				Vector<String> vecIds = mapCorpusIdToParsedId.get( maxMatchCorpusNEId );
				if (vecIds == null) {
					vecIds = new Vector<String>();
					mapCorpusIdToParsedId.put( maxMatchCorpusNEId, vecIds );
				}
				vecIds.add( maxMatchParsedNEId );
				//Keep a record of the inversed mapping too, i.e., parseId-to-corpusVecIds
				vecIds = mapParsedIdToCorpusId.get( maxMatchParsedNEId );
				if (vecIds == null) {
					vecIds = new Vector<String>();
					mapParsedIdToCorpusId.put( maxMatchParsedNEId, vecIds );
				}
				vecIds.add( maxMatchCorpusNEId );
			}
		} //end inner for - j
	} //end outer for - i
     }

     /**
      * A method that retrieves all the sentences from a document.
      * @param doc The document to be traversed
      * @return Vector The list of all sentences
      */
     private Vector getSentences( Document doc ) {
/***********************************************/

     }

     /**
      * Matching Criterion: return the number of overlapping characters between the two NEs
      * plus 4 if the parsed NE head is part of the min_words attribute of the corpus NE.
      * @param corpusNE Hand-annotated NE
      * @param parsedNE Automatically parsed NE 
      * @return int How many characters match (if any)
      */
     private int matchNEs( Element corpusNE, Element parsedNE,
			   Map<String, NEPosition> corpusNeIdToPosition,
			   Map<String, NEPosition> parsedNeIdToPosition ) {
	int result = 0;
	String minWords = mapNeIdToMinWords.get( corpusNE.getAttribute( "id" ) );
	String parsedHead = getNeHead( parsedNE );
	//The following line is only for MUC
	if( parsedHead != null )
		if( minWords != null && minWords.indexOf( parsedHead ) >= 0 ) result += 4; //this will favour a match

	int corpusNEStartIndx = corpusNeIdToPosition.get( corpusNE.getAttribute( "id" ) ).getStartIndex();
	int corpusNEEndIndx = corpusNeIdToPosition.get( corpusNE.getAttribute( "id" ) ).getEndIndex();
	int parsedNEStartIndx = parsedNeIdToPosition.get( parsedNE.getAttribute( "id" ) ).getStartIndex();
	int parsedNEEndIndx = parsedNeIdToPosition.get( parsedNE.getAttribute( "id" ) ).getEndIndex();

	//Get the number of overlapping characters
	if( corpusIndxStart < parsedIndxEnd && corpusIndxEnd > parsedIndxStart ) { //if there is overlap
		result += corpusNEEndIndx - corpusNEStartIndx + 
			  parsedNEEndIndx - parsedNEStartIndx -
			  Math.abs( corpusNEStartIndx - parsedNEStartIndx ) -
			  Math.abs( corpusNEEndIndx - parsedNEEndIndx );
	}

	return result;
     }

     /**
      * Matching Criterion: return the number of words common to both NEs plus 4 if the parsed NE head is part of
      * the min_words attribute of the corpus NE.
      * @param corpusNE Hand-annotated NE
      * @param parsedNE Automatically parsed NE 
      * @return int How many words match (if any)
      */
     private int matchNEs( Element corpusNE, Element parsedNE ) {
	int result = 0;
	String minWords = mapNeIdToMinWords.get( corpusNE.getAttribute( "id" ) );
	String parsedHead = getNeHead( parsedNE );
//System.out.println("minWords: " + minWords + " from " + corpusNE.getAttribute( "id" ));
//System.out.println("parsedHead: " + parsedHead + " from " + parsedNE.getAttribute( "id" ));
	if( parsedHead != null )
		if( minWords != null && minWords.indexOf( parsedHead ) >= 0 ) result += 4; //this will favour a match
	//Get the words composing corpusNe ignoring embedded NEs
	Vector vWords = IOXMLUtils.getElementsByTagNameNoEmbedding( corpusNE, IOXMLUtils.NP_TAG, IOXMLUtils.WORD_TAG );
	StringBuffer wordsCorpusNE = new StringBuffer();
        Node wordNode = null;
        for ( int i = 0; i < vWords.size(); i++ ) {
            wordNode = ( Node )vWords.elementAt( i );
            wordsCorpusNE.append( wordNode.getFirstChild().getNodeValue().trim().toLowerCase() + " ");
        }
	String corpusWords = wordsCorpusNE.toString();
	//Get the words composing parsedNe ignoring embedded NEs
	vWords = IOXMLUtils.getElementsByTagNameNoEmbedding( parsedNE, IOXMLUtils.NP_TAG, IOXMLUtils.WORD_TAG );
        for ( int i = 0; i < vWords.size(); i++ ) {
            wordNode = ( Node )vWords.elementAt( i );
	    String parsedWord = wordNode.getFirstChild().getNodeValue().trim().toLowerCase();
	    int idx = corpusWords.indexOf( parsedWord );
	    boolean matched = false;
	    //The following should avoid matches like the -> their
	    //matched if whole word has been matched (i.e., char after matched string is space)
	    if( idx >= 0 )
		    matched = (corpusWords.length() <= idx + parsedWord.length()) || Character.isSpaceChar( corpusWords.charAt( idx + parsedWord.length() ) );
	    if( matched ) result++;
        }

	return result;
     }

     /**
      * Retrieves the head of a parsed NE, which has been processed by the Syntactic Heuristics.
      * @param ne The <ne> node
      * @return String The head itself
      */
     private String getNeHead( Node ne ) {
	String result = null;
        Vector nl = IOXMLUtils.getElementsByTagNameNoEmbedding( ne, IOXMLUtils.NP_TAG, IOXMLUtils.NP_HEAD_TAG );
        if (nl.size() > 0) {
		Node headNode = (Node)nl.elementAt( 0 ); //There is only one head
	        result = getNePhrase( (Element) headNode );
	}
	return result;
     }

     /**
      * Retrieves the phrase of an NE.
      * @param ne The <ne> node
      * @return String The phrase
      */
     private String getNePhrase( Element element ) {
	NodeList words = element.getElementsByTagName(IOXMLUtils.WORD_TAG);
        StringBuffer sb = new StringBuffer();
        Node wordNode = null;
        for (int i = 0; i < words.getLength(); i++) {
            wordNode = words.item(i);
            sb.append(wordNode.getFirstChild().getNodeValue().trim() + " ");
        }

	return ( sb.substring(0, sb.length() - 1) ); //Ignore last white space
     }


     /**
      * Main method takes a file name as a parameter or no parameters, in which case file names are read from standard input.
      */
     public static void main ( String args[] ) throws IOException {
	if( args.length == 0 ) {
		BufferedReader in
			= new BufferedReader( new InputStreamReader( System.in ) );
		String fileName = in.readLine();
		while ( fileName != null ) {
			System.out.println("File name: " + fileName );
			NeAlignerGNM nea = new NeAlignerGNM();
			nea.processFile( fileName );
			fileName = in.readLine();
		}
	}
	else {
		NeAlignerGNM nea = new NeAlignerGNM();
		nea.processFile( args[0] );
	} //end if-else
     }


     /***** PRIVATE CLASS **********/

     private class NEPosition {
	int startIndex = -1;
	int endIndex = -1;

	public int getStartIndex() { return startIndex; }
	public int getEndIndex() { return endIndex; }
	public int setStartIndex( int indx ) { startIndex = indx; }
	public int setEndIndex( int indx ) { endIndex = indx; }
     }

} //end class NeAlignerGNM
