/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR;

//For DOM Tree manipulation
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

//Others
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.Local;
import java.util.Vector;
import java.util.Map;
import java.util.HashMap;

/**
 * A class that contains static methods used to align a list of nes marked-up by a chunker
 * with a list of nes marked-up by human subjects (from a corpus).
 */
public class NeAlignment {

    /**
     * Returns a neId-to-vector mapping, used to translate a given neId from the corpus to a vector
     * of matching nes from a parsed file.
     * @param documentCorpus The DOM document containing the Corpus annotation
     * @param documentParsed The DOM document containing the automatically generated annotation
     */
    public static Map constructCorpusIdToParsedId(Document documentCorpus, Document documentParsed) {
        Local.out.println("BUILDING THE CORPUSID_TO_PARSEDID MAP...");
        Map<String, Vector<String>> result = new HashMap<String, Vector<String>>();
        NodeList corpusNes = documentCorpus.getElementsByTagName("ne");
        NodeList parsedNes = documentParsed.getElementsByTagName("ne");
        int counterCorpusNes = 0, counterParsedNes = 0;
        int lastIndexCorpus = -1;
        int lastIndexParsed = -1;
        int numberOfWords = 0;
        int maximumStayCounter = 0;
        while (counterCorpusNes < corpusNes.getLength()) {
            Element corpusNe = (Element) corpusNes.item(counterCorpusNes);
            String corpusNeId = corpusNe.getAttribute(IOXMLUtils.ID_PROPERTY_NAME);
            Element parsedNe = null;
            boolean match = false;
            int previousCounter = counterParsedNes;
            while (!match
                   && counterParsedNes < parsedNes.getLength()
                   && (counterParsedNes - previousCounter) < 3) {
                parsedNe = (Element) parsedNes.item(counterParsedNes);
                Local.out.println("Trying to match: " + corpusNeId + "::" + parsedNe.getAttribute(IOXMLUtils.ID_PROPERTY_NAME));
                Local.out.println("indexes: " + lastIndexCorpus + "::" + lastIndexParsed + "::" + maximumStayCounter);
                int indexes[] = lastMatchedWords(corpusNe, parsedNe, lastIndexCorpus, lastIndexParsed);
                if (indexes[0] > 0) { //if number of words matched is greater than 0
                    match = true;
                    numberOfWords += indexes[0];
                    lastIndexCorpus = indexes[1];
                    lastIndexParsed = indexes[2];
                } else {
                    if (maximumStayCounter != 0) previousCounter++;
                    counterParsedNes++;
                    lastIndexParsed = -1; //reset index
                    numberOfWords = 0;
                    maximumStayCounter = 0;
                }
                Local.out.println(match);
            }
            if (match) {
                if (maximumStayCounter < 2 && (lastIndexParsed < parsedNe.getElementsByTagName(IOXMLUtils.WORD_TAG).getLength() - 1
                                               || numberOfWords < parsedNe.getElementsByTagName(IOXMLUtils.WORD_TAG).getLength())) { //There are still words to be matched
                    if (numberOfWords - lastIndexParsed <= 0) { //Means Words have been skipped
                        lastIndexParsed = -1;
                        maximumStayCounter++;
                    }
                    counterCorpusNes++;
                    lastIndexCorpus = -1;
                }
                //else if( lastIndexCorpus < corpusNe.getElementsByTagName( IOXMLUtils.WORD_TAG ).getLength() - 1 ) {
                else if (lastIndexCorpus < IOXMLUtils.getElementsByTagNameNoEmbedding(corpusNe, corpusNe.getNodeName(), IOXMLUtils.WORD_TAG).size() - 1) {
                    counterParsedNes++;
                    lastIndexParsed = -1;
                    numberOfWords = 0;
                    maximumStayCounter = 0;
                } else {
                    counterCorpusNes++;
                    counterParsedNes++;
                    lastIndexCorpus = -1;
                    lastIndexParsed = -1;
                    numberOfWords = 0;
                    maximumStayCounter = 0;
                }
                String parsedNeId = parsedNe.getAttribute(IOXMLUtils.ID_PROPERTY_NAME);
                //previous version -- result.put( corpusNeId, parsedNeId );
                Vector<String> vecIds = (Vector<String>) result.get(corpusNeId);
                if (vecIds == null) {
                    vecIds = new Vector<String>();
                    result.put(corpusNeId, vecIds);
                }
                vecIds.add(parsedNeId);
            } else {
                counterParsedNes = previousCounter; //return counter
                counterCorpusNes++;
                lastIndexCorpus = -1;
            }
        } //end outer while
        return result;
    }


    /**
     * A private method used by method constructCorpusIdToParsedId() that returns an array of
     * three integers: 1- number of words matched, 2- index of the last word matched from the corpus ne,
     * and 3- index of the last word matched from the parsed ne.
     * @param corpusNe A pointer to the DOM node holding the corpus ne to be matched
     * @param parsedNe A pointer to the DOM node holding the parsed ne to be matched
     * @param lastIndexCorpus previous index of the last word matched from the corpus ne
     * @param lastIndexParsed previous index of the last word matched from the parsed ne
     */
    private static int[] lastMatchedWords(Element corpusNe, Element parsedNe, int lastIndexCorpus, int lastIndexParsed) {
        int result[] = new int[3];
        int indexParsed = lastIndexParsed;
        int indexCorpus = lastIndexCorpus;
        Vector corpusNeWords = IOXMLUtils.getElementsByTagNameNoEmbedding(corpusNe, corpusNe.getNodeName(), IOXMLUtils.WORD_TAG);
        Vector parsedNeWords = IOXMLUtils.getElementsByTagNameNoEmbedding(parsedNe, parsedNe.getNodeName(), IOXMLUtils.WORD_TAG);
        Vector corpusWordsText = IOXMLUtils.toStringsVectorKeepLetterCase(corpusNeWords);
        Vector parsedWordsText = IOXMLUtils.toStringsVectorKeepLetterCase(parsedNeWords);
        Local.out.println("corpus: " + corpusWordsText);
        Local.out.println("parsed: " + parsedWordsText);
        int numberOfWordsMatched = 0;
        for (int i = lastIndexCorpus + 1; i < corpusWordsText.size(); i++) {
            int auxParsed = parsedWordsText.indexOf((String) corpusWordsText.elementAt(i), indexParsed + 1);
            if (auxParsed >= 0) { //if new match found store new indexes
                indexParsed = auxParsed;
                indexCorpus = i;
                numberOfWordsMatched++;
            }
        }
        result[0] = numberOfWordsMatched;
        if (numberOfWordsMatched > 0) {
            result[1] = indexCorpus;
            result[2] = indexParsed;
        } else {
            result[1] = lastIndexCorpus;
            result[2] = lastIndexParsed;
        }
        Local.out.println("indexes: " + indexCorpus + "::" + indexParsed);
        return result;
    }


} //end class NeAlignment
