/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR;

import uk.ac.essex.malexa.nlp.dp.GuiTAR.la.Sense;

/**
 * A class that encapsulates the general functionality of a Property (of a Discourse Entity).
 * @author Mijail A. Kabadjov
 * @version 1.1
 */
public class Property {

	//Constants
	public static final byte TYPE = 1;
	public static final byte ATTRIBUTE = 2;
	public static final byte RELATION = 3;
	public static final byte NAME = 3;


	//Instance variables
	private byte		type;		//Range of valid values : {type, attribute, relation}
	private Sense		predicate;	//type=type/attr -> predicate=sense, otherwise predicate="the relation" (e.g. of)
	private DiscourseEntity	argument;	//only when type=relation, points to another DE


	public Property ( byte tp, Sense pred ) {
		type = tp;
		predicate = pred;
	}

	public Property ( Sense pred, DiscourseEntity arg ) {
		type = Property.RELATION; //This is for relations only
		predicate = pred;
		argument = arg;
	}


	public byte getType() {
		return type;
	}

	public Sense getPredicate() {
		return predicate;
	}

	public DiscourseEntity getArgument() {
		return argument;
	}

	public boolean equals( Object o ) {
		return ( getPredicate().equals( ((Property)o).getPredicate() ) );
	}

	/** It seems that this method has to be overriden as well, in order for the the set framework to work ***/
	public int hashCode() {
		return ( getPredicate().hashCode() );
	}


	public String toString() {
		return( "Pty-" + type + "-" + predicate );
	}

} //end class Property

