/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR;

import org.w3c.dom.*;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.*;

/**
 * A class that encapsulates the general functionality of an Utterance.
 * Eventually this class might have additional functionality, but for the moment it acts as a Segment,
 * the main difference being that its children are Cfs not Utterances nor Segments.
 * @author Mijail A. Kabadjov
 * @version 1.1
 */
public class Utterance extends Segment {

    /**
     * Initialises the instance variables.
     * @param node The DOM node of this utterance
     * @param seg The segment to which this utterance belongs
     */
    public Utterance(Node node, Segment seg) {
        super(node, seg);
    }


    /**
     * Adds a new Cf to this utterance.
     * @param child The Cf to be added
     */
    public void addChild(Cf child) {
        children.add(child);
    }


    /**
     * Overrides method in class Segment to enforce that an utterance cannot contain
     * other utterances nor segments.
     * @param child The child (Utterance/Segment) to be added
     */
    public void addChild(Segment child) {
        Local.out.println("*** ERROR *** An attempt to attach a Segment/Utterance to an Utterance was made!");
    }

} //end class Utterance
