/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.eval;

import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;

import java.util.Set;

import uk.ac.essex.malexa.nlp.dp.GuiTAR.IOXMLUtils;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.DiscourseModel;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.DiscourseModelImplementer;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.Local;

/**
 * A simple class that takes a file previously processed by GuiTAR and generates an HTML file that displays
 * the original text with members of coreference chains in a different color. Any given member of a coreference
 * chain has a unique color and a unique index displayed as a superscript. Additionally, markables are underlined.
 */
public class GenerateErrorsInHTML {

    public void transformFile( String filename, String anteTag, String refAnteTag ) {
        Document domDocument = IOXMLUtils.load( filename ); //Load XML file
        transformDomDocument( domDocument, filename, anteTag, refAnteTag );
    }


    public void transformDomDocument( Document domDocument, String filename, String anteTag, String refAnteTag ) {
        //Build flat discourse models - coreference chains
        DiscourseModel dm = new DiscourseModelImplementer( domDocument, anteTag );
        DiscourseModel dmRef = new DiscourseModelImplementer( domDocument, refAnteTag );
//Local.out.println( "Number of entities: " + dm.getNumberOfEntities() );
//System.exit(0);
        //HTML template
        Local.out.println( "<html>" );
        Local.out.println( "<head>" );
        Local.out.println( "<title>AR Error Highlight</title>" );
        Local.out.println( "<p><center><h2>AR Error Highlight<br>for<br>" + filename + "</h2></center></p>" );
        Local.out.println( "</head>" );
        Local.out.println( "<body>" );
        Local.out.println( "<br>" );
        Local.out.println( "<br>" );
        Local.out.println( "<H3>ANNOTATION USED:</H3>" );
        Local.out.println( "<table border=3 cellsize=3 cellpadding=3>" );
        Local.out.println( "<tr><td><B>Annotation</B></td><td><B>Meaning</B></td></tr>" );
        Local.out.println( "<tr><td><font color=\"red\">Red</font></td><td>Wrong Matches (WM)</td></tr>" );
        Local.out.println( "<tr><td><font color=\"maroon\">Maroon</font></td><td>Spurious Matches (SM)</td></tr>" );
        Local.out.println( "<tr><td><font color=\"aqua\">Aqua</font></td><td>No Matches (NM)</td></tr>" );
        Local.out.println( "<tr><td><font color=\"green\">Green</font></td><td>Correctly Resolved Anaphor</td></tr>" );
        Local.out.println( "<tr><td><font color=\"blue\">[] Blue</font> square brackets</td><td>Delimit markables</td></tr>" );
        Local.out.println( "<tr><td><SUB>(3)</SUB>Sub-index in round brackets same colour as markable</td><td>Resolved coreference chain unique id</td></tr>" );
        Local.out.println( "<tr><td><font color=\"yellow\"><SUB>(0)</SUB></font>Sub-index in round brackets <font color=\"yellow\">Yellow</font></td><td>Reference Annotation coreference chain unique id</td></tr>" );
        Local.out.println( "</table>" );
        Local.out.println( "<br>" );
        Local.out.println( "<SUP>*</SUP>Note: Antecedents of resolved anaphors are not coloured." );
        Local.out.println( "<br>" );
        Local.out.println( "<br>" );
        Local.out.println( "<br>" );
        //Read XML and produce HTML content
        generateHTMLContent( dm, dmRef, domDocument.getDocumentElement() );
        //End template
        Local.out.println( "</body>" );
        Local.out.println( "</html>" );
    }


    private void generateHTMLContent( DiscourseModel dm, DiscourseModel dmRef, Node node ) {
        if( !node.hasChildNodes() ) {
            if( node.getNodeName().equalsIgnoreCase( "#text" ) )
                Local.out.print( node.getNodeValue().trim() + " " );
        }
        else {
            if( node.getNodeName().equalsIgnoreCase( IOXMLUtils.NP_TAG ) ) {
                String neId = ( ( Element ) node ).getAttribute( IOXMLUtils.ID_PROPERTY_NAME );
                String color = "#000000"; //default Black
                //Get antecedent proposed by algorithm
                Object proposedAnte = dm.getAntecedent( neId );
                if( dmRef.isAnaphoric( neId ) ) {
                    //Get original equivalence class
                    Set eqClass = dmRef.getEquivalenceClass( neId );
                    if( eqClass.contains( proposedAnte ) ) color = "green"; //correctly resolved Anaphors Green
                    else if( proposedAnte != null ) color = "red"; //Wrong matches Red
                    else color = "aqua"; //No matches Light Blue
                }
                else if( dm.isAnaphoric( neId ) )
                    color = "maroon"; //Spurious matches

                Local.out.println( "<font color=\"" + color + "\">" );
                //Local.out.println("<U>");
                Local.out.println( "<font color=\"blue\">[</font>" ); //Blue
                //Get children for pre-order traversal
                NodeList children = node.getChildNodes();
                int numberOfChildren = children.getLength();
                for( int i = 0; i < numberOfChildren; i++ )
                    generateHTMLContent( dm, dmRef, children.item( i ) );
                Local.out.println( "<font color=\"blue\">]</font>" ); //Blue
                int classIdx = dm.getEquivalenceClassIndex( neId );
                if( classIdx != -1 ) Local.out.print( "<SUB>(" + classIdx + ")</SUB>" );
                int classIdxRef = dmRef.getEquivalenceClassIndex( neId );
                if( classIdxRef != -1 ) Local.out.print( "<font color=\"yellow\"><SUB>(" + classIdxRef + ")</SUB></font>" ); //Reference annotation in yellow
                //Local.out.println("</U>");

                Local.out.println( "</font>" );
            }
            else if( node.getNodeName().equalsIgnoreCase( "p" ) ) {
                Local.out.println( "<p>" );
                //Get children for pre-order traversal
                NodeList children = node.getChildNodes();
                int numberOfChildren = children.getLength();
                for( int i = 0; i < numberOfChildren; i++ )
                    generateHTMLContent( dm, dmRef, children.item( i ) );
                Local.out.println( "</p>" );
            }
            else if( !node.getNodeName().equalsIgnoreCase( "annoinfo" ) ) { //ignore annotator information
                //Get children for pre-order traversal
                NodeList children = node.getChildNodes();
                int numberOfChildren = children.getLength();
                for( int i = 0; i < numberOfChildren; i++ )
                    generateHTMLContent( dm, dmRef, children.item( i ) );
            }
        }
    }


    public static void main( String args[] ) {
        GenerateErrorsInHTML g = new GenerateErrorsInHTML();
        if( args.length == 3 )
            g.transformFile( args[0], args[1], args[2] );
        else {
            Local.out.println( "\n*** Invalid parameters! \nUsage: java GenerateErrorsInHTML fileName anteTagName refAnteTagName" );
            System.exit( 1 );
        }
    }

} //end GenerateErrorsInHTML
