/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.*;


/**
 * <b>Description:</b> Used for the GuiTAR tool's GUI.<br>
 * <b>Copyright:</b> Copyright (c) 2005<br>
 * <b>Company:</b> Essex University<br>
 * @author Philippe J. Goux
 * @version 1.0
 */

public class AboutDialog extends JDialog implements Gui {
    // ===========================================================================
    // CONSTANTS
    private static final String ID = "About Dialog";


    // ===========================================================================
    // VARIABLES
    public JButton button = new JButton();
    private String comments = Language.getDescription() +
                              Language.getTranslator() +
                              Local.NEWLINE +
                              Language.getUsage();


    // ===========================================================================
    // CONSTRUCTORS
    public AboutDialog(Frame parent) {
        super(parent);
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try {
            initialiseGUI();
            initialiseHistory();
            initialiseLanguage();
            addListeners();
        } catch (Exception e) {
            Local.err.printException(e);
        }
    }


    public AboutDialog() {
        this(null);
    }


    // ===========================================================================
    // PUBLIC FUNCTIONS

    //Overridden so we can exit when window is closed
    protected void processWindowEvent(WindowEvent e) {
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            dispose();
        }
        super.processWindowEvent(e);
    }


    // ===========================================================================
    // PRIVATE FUNCTIONS
    private JPanel createTopPanel() {
        JPanel topPanel = new JPanel();
        topPanel.setLayout(new BorderLayout());
        topPanel.add(createDescriptionPanel(), BorderLayout.CENTER);
        topPanel.add(createImagePanel(), BorderLayout.WEST);
        return topPanel;
    }


    private JPanel createDescriptionPanel() {
        JPanel descriptionPanel = new JPanel(new BorderLayout());
        descriptionPanel.setBorder(BorderFactory.createEmptyBorder(10, 60, 10, 10));

        // Delete this, its used just to get font and color!!!
        JLabel labelCompany = new JLabel();
        //----------------------------------------------------

        JTextArea textAreaDescription = new JTextArea(comments);
        textAreaDescription.setEditable(false);
        textAreaDescription.setFont(labelCompany.getFont());
        textAreaDescription.setBackground(labelCompany.getBackground());
        descriptionPanel.add(textAreaDescription, BorderLayout.CENTER);
        return descriptionPanel;
    }


    private JPanel createImagePanel() {
        JPanel imagePanel = new JPanel();
        imagePanel.setLayout(new FlowLayout());
        imagePanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        JLabel imageLabel = new JLabel();
        imageLabel.setIcon(Utils.getImageIcon("guitar_large.png"));
        imagePanel.add(imageLabel, null);
        return imagePanel;
    }


    private JPanel createButtonPanel() {
        JPanel buttonPanel = new JPanel();
        button.setPreferredSize(new Dimension(90, 25));
        buttonPanel.add(button, null);
        buttonPanel.setLayout(new FlowLayout());
        return buttonPanel;
    }


    // ===========================================================================
    // INTERFACE IMPLEMENTATIONS
    public void addListeners() {
        button.addActionListener(new ActionListener() {
            //Close the dialog on a button event
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
    }


    public void initialiseGUI() throws Exception {
        this.setResizable(false);

        JPanel mainPanel = new JPanel();
        this.getContentPane().add(mainPanel, null);
        mainPanel.setLayout(new BorderLayout());
        mainPanel.add(createTopPanel(), BorderLayout.NORTH);
        mainPanel.add(createButtonPanel(), BorderLayout.SOUTH);
    }


    public String getID() {
        return ID;
    }


    public void initialiseHistory() {
    }


    public void initialiseLanguage() {
        this.setTitle(Language.TITLE_ABOUT + " GuiTAR");
        button.setText(Language.BUTTON_OK);
    }
}
