/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;

import javax.swing.*;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.*;

/**
 * <b>Description:</b> Used for the GuiTAR tool's GUI.<br>
 * <b>Copyright:</b> Copyright (c) 2005<br>
 * <b>Company:</b> Essex University<br>
 * @author Philippe J. Goux
 * @version 1.0
 */

public class BrowsePanel extends JPanel implements Gui {
    // ===========================================================================
    // CONSTANTS
    private final String ID;


    // ==========================================================================
    // VARIABLES
    public static final String SEPARATOR = " ";

    public JLabel label = new JLabel();
    public JTextField textField = new JTextField();
    public JButton button = new JButton();

    private byte[] filterTypes = new byte[] {FileBrowser.FILTER_ACCEPT_ALL_FILES};
    private boolean multiSelectionEnabled = false;
    private java.util.List<File> selectedFiles = new ArrayList<File>();


    // ==========================================================================
    // CONTRUCTORS
    public BrowsePanel(String label, boolean multiSelectionEnabled, String id) {
        ID = id;
        this.multiSelectionEnabled = multiSelectionEnabled;

        try {
            initialiseGUI();
            this.label.setText(label);

            initialiseHistory();
            initialiseLanguage();
            addListeners();
        } catch (Exception ex) {
            Local.err.printException(ex);
        }
    }


    // ==========================================================================
    // PUBLIC FUNCTIONS
    public void setFilterTypes(byte[] filterTypes) {
        this.filterTypes = filterTypes;
    }


    public boolean getMultiSelectionEnabled() {
        return multiSelectionEnabled;
    }


    public java.util.List getSelectedFiles() {
        return selectedFiles;
    }


    // ==========================================================================
    // PRIVATE FUNCTIONS
    private void browse(ActionEvent e) {
        File[] results = FileBrowser.showOpenDialog(MainFrame.mainFrame, multiSelectionEnabled, filterTypes);
        if (results != null) {
            String path = "";
            String[] history = new String[results.length];
            selectedFiles.clear();
            for (int i = 0; i < results.length; i++) {
                history[i] = results[i].getPath();
                File o = null;
                if (multiSelectionEnabled)
                    o = new FileObject(results[i]);
                else
                    o = results[i];
                selectedFiles.add(o);
                path += "\"" + o.toString() + "\"" + ((i + 1 < results.length) ? SEPARATOR : "");

            }
            textField.setText(path);
            History.setBrowsePanelFiles(ID, history);
        }
    }


    // ===========================================================================
    // INTERFACE IMPLEMENTATIONS
    public void addListeners() {
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                browse(e);
            }
        });
    }


    public void initialiseGUI() throws Exception {
        button.setIcon(Utils.getImageIcon("open16.gif"));
        this.setPreferredSize(new Dimension(200, 30));
        this.setMinimumSize(new Dimension(200, 30));
        this.setLayout(new GridBagLayout());
        this.add(button, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
                                                GridBagConstraints.NONE, new Insets(0, 1, 0, 0), 0, 0));
        this.add(textField, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER,
                GridBagConstraints.HORIZONTAL, new Insets(0, 1, 0, 1), 0, 0));
        this.add(label, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE, new Insets(0, 0, 0, 1), 0, 0));
    }


    public String getID() {
        return ID;
    }


    public void initialiseHistory() {
        String[] filePaths = History.getBrowsePanelFiles(ID, new String[0]);
        for (int i = 0; i < filePaths.length; i++) {
            File file = null;
            if (multiSelectionEnabled)
                file = new FileObject(filePaths[i]);
            else
                file = new File(filePaths[i]);
            if (file.exists()) {
                textField.setText(textField.getText() + "\"" + file.toString() + "\"" + SEPARATOR);
                selectedFiles.add(file);
            }
        }
    }


    public void initialiseLanguage() {
        button.setText(Language.BUTTON_BROWSE);
    }
}
