/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.*;

/**
 * <b>Description:</b> Used for the GuiTAR tool's GUI.<br>
 * <b>Copyright:</b> Copyright (c) 2005<br>
 * <b>Company:</b> Essex University<br>
 * @author Philippe J. Goux
 * @version 1.0
 */
public class CheckBoxPanel extends JPanel implements Gui {
    // ===========================================================================
    // CONSTANTS
    private final String ID;

    // ===========================================================================
    // VARIABLES
    public JCheckBox checkBox = new JCheckBox();
    public JTextField textField = new JTextField();

    // ===========================================================================
    // CONSTRUCTORS
    public CheckBoxPanel(String label, String id) {
        ID = id;
        try {
            initialiseGUI();
            checkBox.setText(label);

            initialiseHistory();
            initialiseLanguage();
            addListeners();
        } catch (Exception ex) {
            Local.err.printException(ex);
        }
    }


    // ===========================================================================
    // PUBLIC FUNCTIONS
    public boolean isSelected() {
        boolean selected = checkBox.isSelected();
        History.setCheckboxPanelSelected(ID, selected);
        return selected;
    }


    public String getValue() {
        String value = textField.getText();
        History.setCheckboxPanelValue(ID, value);
        return value;
    }


    public String[] getValues() {
        String value = textField.getText();
        History.setCheckboxPanelValue(ID, value);
        return value.split("\\s");
    }


    // ===========================================================================
    // PRIVATE FUNCTIONS


    // ===========================================================================
    // INTERFACE IMPLEMENTATIONS
    public void addListeners() {
        checkBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                textField.setEnabled(checkBox.isSelected());
            }
        });
    }


    public void initialiseGUI() throws Exception {
        checkBox.setSelected(false);
        textField.setEnabled(false);
        this.setLayout(new GridBagLayout());
        this.add(checkBox, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                                  GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 1), 0, 0));
        this.add(textField, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0,
                GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 1, 0, 0), 0, 0));
    }


    public String getID() {
        return ID;
    }


    public void initialiseHistory() {
        boolean selected = History.getCheckboxPanelSelected(ID,false);
        checkBox.setSelected(selected);
        textField.setEnabled(selected);
        textField.setText(History.getCheckboxPanelValue(ID, ""));
    }


    public void initialiseLanguage() {
    }
}
