/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.gui;

import java.awt.Component;
import java.io.File;
import javax.swing.JFileChooser;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.Local;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.Language;

/**
 * <b>Description:</b> Used for the GuiTAR tool's GUI.<br>
 * <b>Copyright:</b> Copyright (c) 2005<br>
 * <b>Company:</b> Essex University<br>
 * @author Philippe J. Goux
 * @version 1.0
 */

public abstract class FileBrowser {
    // ==========================================================================
    // VARIABLES
    public static final byte FILTER_ACCEPT_ALL_FILES = 0;
    public static final byte FILTER_ACCEPT_DIRECTORIES = 1;
    public static final byte FILTER_ACCEPT_IMAGES = 2;
    public static final byte FILTER_ACCEPT_DATAFILES = 3;
    public static final byte FILTER_ACCEPT_XML_FILES = 4;
    public static final String SEPARATOR = ";";

    private static String directory = "";


    // ==========================================================================
    // PUBLIC FONCTIONS
    public static File[] showOpenDialog(Component mainFrame, boolean multiSelectionEnabled, byte[] filterTypes) {
        // Intitialyze function
        if (directory.equals(""))
            directory = System.getProperty("user.dir");
        JFileChooser fileBrowser = new JFileChooser(directory);
        fileBrowser.setMultiSelectionEnabled(multiSelectionEnabled);
        fileBrowser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fileBrowser.setAcceptAllFileFilterUsed(false);

        // Set Filter files
        for (int i = 0; i < filterTypes.length; i++) {
            if (filterTypes[i] == FILTER_ACCEPT_DIRECTORIES) {
                if (filterTypes.length < 2)
                    fileBrowser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                else
                    fileBrowser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
                fileBrowser.addChoosableFileFilter(new DirectoryFilter());
            } else if (filterTypes[i] == FILTER_ACCEPT_IMAGES) {
                fileBrowser.addChoosableFileFilter(new ImageFilter());
            } else if (filterTypes[i] == FILTER_ACCEPT_DATAFILES) {
                fileBrowser.addChoosableFileFilter(new DataFilesFilter());
            } else if (filterTypes[i] == FILTER_ACCEPT_XML_FILES) {
                fileBrowser.addChoosableFileFilter(new XMLFilesFilter());
            } else {
                fileBrowser.addChoosableFileFilter(new AllFilesFilter());
            }
        }
        fileBrowser.setFileFilter(fileBrowser.getChoosableFileFilters()[0]);

        // Show JFileChooser
        int returnVal = fileBrowser.showOpenDialog(mainFrame);

        // Get selected files
        File[] files = null;
        if (returnVal == JFileChooser.APPROVE_OPTION && multiSelectionEnabled) {
            files = fileBrowser.getSelectedFiles();
        } else if (returnVal == JFileChooser.APPROVE_OPTION && !multiSelectionEnabled) {
            files = new File[] {fileBrowser.getSelectedFile()};
        }
        if (files != null) {
            if (files[0].isFile())
                directory = files[0].getParentFile().getPath();
            else
                directory = files[0].getPath();
        }
        return files;
    }


    // ==========================================================================
    // PRIVATE FONCTIONS
    private static String getFileExtension(File file) {
        String extention = null;
        String name = file.getName();

        int i = name.lastIndexOf(".");
        if (i > 0 && i < name.length() - 1)
            extention = name.substring(i + 1).toLowerCase();

        return extention;
    }


    // ==========================================================================
    // PRIVATE FILTER CLASSES
    private static class ImageFilter extends javax.swing.filechooser.FileFilter {
        public boolean accept(File file) {
            // Accept gif, jpg, and png files, and display directories.
            if (file.isDirectory())
                return true;

            String extension = getFileExtension(file);
            if (extension != null &&
                (extension.equalsIgnoreCase("gif") ||
                 extension.equalsIgnoreCase("jpeg") ||
                 extension.equalsIgnoreCase("jpg") ||
                 extension.equalsIgnoreCase("png"))) {
                return true;
            }
            return false;
        }


        public String getDescription() {
            return Language.DESCRIPTION_IMAGES_ONLY + " *.gif; *.jpeg; *.jpg; *.png)";
        }
    }


    private static class DataFilesFilter extends javax.swing.filechooser.FileFilter {
        public boolean accept(File file) {
            // Accept txt and dat files, and display directories.
            if (file.isDirectory())
                return true;

            String extension = getFileExtension(file);
            if (extension != null &&
                (extension.equalsIgnoreCase("dat") ||
                 extension.equalsIgnoreCase("ini") ||
                 extension.equalsIgnoreCase("txt"))) {
                return true;
            }
            return false;
        }


        public String getDescription() {
            return Language.DESCRIPTION_DATA_FILES + " (*.ini; *.dat; *.txt)";
        }
    }


    private static class DirectoryFilter extends javax.swing.filechooser.FileFilter {
        public boolean accept(File file) {
            // Accept directories only.
            if (file.isDirectory())
                return true;
            return false;
        }


        public String getDescription() {
            return Language.DESCRIPTION_DIRECTORIES_ONLY;
        }
    }


    private static class AllFilesFilter extends javax.swing.filechooser.FileFilter {
        public boolean accept(File file) {
            // Accept all files.
            return true;
        }


        public String getDescription() {
            return Language.DESCRIPTION_ALL_FILES + " (*.*)";
        }
    }


    private static class XMLFilesFilter extends javax.swing.filechooser.FileFilter {
        public boolean accept(File file) {
            // Accept directories.
            if (file.isDirectory())
                return true;

            // Accept XML files.
            String extension = getFileExtension(file);
            if (extension != null &&
                (extension.equalsIgnoreCase("xml"))) {
                return true;
            }

            return false;
        }


        public String getDescription() {
            return Language.DESCRIPTION_XML_FILES + " (*.xml)";
        }
    }
}

