/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.gui;

import java.io.File;

/**
 * <b>Description:</b> Used for the GuiTAR tool's GUI.<br>
 * <b>Copyright:</b> Copyright (c) 2005<br>
 * <b>Company:</b> Essex University<br>
 * @author Philippe J. Goux
 * @version 1.0
 */

public class FileObject extends File {
    // ===========================================================================
    // CONSTANTS

    // ===========================================================================
    // VARIABLES
    private File html = null;
    private File processed = null;

    // ===========================================================================
    // CONSTRUCTORS
    public FileObject(File file, File html, File processed) {
        super(file.getPath());
        this.html = html;
        this.processed = processed;
        initialise();
    }


    public FileObject(File file, File html) {
        this(file, html, null);
    }


    public FileObject(File file) {
        this(file, null);
    }

    public FileObject(String path) {
        this(new File(path));
    }



    // ===========================================================================
    // PUBLIC FUNCTIONS
    public void setProcessed(File file) {
        processed = file;
    }


    public File getProcessed() {
        return processed;
    }


    public void setHtml(File file) {
        this.html = file;
    }


    public File getHtml() {
        return html;
    }


    public String getNameWithoutExtention() {
        String str = "";
        String[] name = this.getName().split("\\.");
        for (int i = 0; i + 1 < name.length; i++)
            str += name[i];
        return str;
    }


    public String toString() {
        return this.getName();
    }


    // ===========================================================================
    // PRIVATE FUNCTIONS
    private void initialise() {
        if (html == null)
            setHtml(new File(this.getNameWithoutExtention() + ".html"));
        if (processed == null)
            setProcessed(new File("processed." + this.getName()));
    }
}
