/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.gui;

import java.awt.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.*;


/**
 * <b>Description:</b> Used for the GuiTAR tool's GUI.<br>
 * <b>Copyright:</b> Copyright (c) 2005<br>
 * <b>Company:</b> Essex University<br>
 * @author Philippe J. Goux
 * @version 1.0
 */

public class OptionPanel extends JPanel implements Gui {
    // ===========================================================================
    // CONSTANTS
    private static final String ID = "Option Panel";

    // ===========================================================================
    // VARIABLES
    private BrowsePanel browsePanel_directory;
    private BrowsePanel browsePanel_tagset;
    private CheckBoxPanel evaluationCheckBoxPanel;
    private CheckBoxPanel logCheckBoxPanel;
    private JCheckBox jCheckBox_parsed;
    private JCheckBox jCheckBox_preprocess;

    // ===========================================================================
    // CONSTRUCTORS
    public OptionPanel() {
        try {
            initialiseGUI();
            initialiseHistory();
            initialiseLanguage();
            addListeners();
        } catch (Exception ex) {
            Local.err.printException(ex);
        }
    }


    // ===========================================================================
    // PUBLIC FUNCTIONS
    public File getDirectory() {
        Iterator i = browsePanel_directory.getSelectedFiles().iterator();
        if (i.hasNext())
            return (File) i.next();
        return null;
    }


    public File getTagSet() {
        Iterator i = browsePanel_tagset.getSelectedFiles().iterator();
        if (i.hasNext())
            return (File) i.next();
        return null;

    }


    public boolean logInformation() {
        return logCheckBoxPanel.isSelected();
    }


    public String[] logTypes() {
        return logCheckBoxPanel.getValues();
    }


    public boolean runPreprocess() {
        History.getOptionPanelPreprocess(ID, jCheckBox_preprocess.isSelected());
        return jCheckBox_preprocess.isSelected();
    }


    public boolean runEvaluation() {
        return evaluationCheckBoxPanel.isSelected();
    }


    public boolean parsedEvaluation() {
        History.setOptionPanelParsed(ID, jCheckBox_parsed.isSelected());
        return jCheckBox_parsed.isSelected();
    }


    public Vector<String> evaluationTypes() {
        Vector<String> evaluationTypes = new Vector<String>();
        String[] types = evaluationCheckBoxPanel.getValues();
        for (int i = 0; i < types.length; i++)
            evaluationTypes.add(types[i]);
        return evaluationTypes;
    }


    // ===========================================================================
    // PRIVATE FUNCTIONS
    private BrowsePanel createBrowsePanel(String label, String tip, String id,  byte[] filterTypes) {
        BrowsePanel browsePanel = new BrowsePanel(label, false, id);
        browsePanel.setFilterTypes(filterTypes);
        browsePanel.textField.setEditable(false);
        browsePanel.textField.setBackground(Color.WHITE);
        browsePanel.label.setToolTipText(tip);
        browsePanel.button.setPreferredSize(new Dimension(120, 25));
        browsePanel.label.setPreferredSize(new Dimension(150, 25));
        browsePanel.button.setMinimumSize(new Dimension(120, 25));
        browsePanel.label.setMinimumSize(new Dimension(150, 25));
        return browsePanel;
    }


    private CheckBoxPanel createCheckBoxPanel(String label, String tip, String id) {
        CheckBoxPanel checkBoxPanel = new CheckBoxPanel(label, id);
        checkBoxPanel.checkBox.setToolTipText(tip);
        checkBoxPanel.checkBox.setPreferredSize(new Dimension(150, 25));
        checkBoxPanel.textField.setPreferredSize(new Dimension(120, 21));
        return checkBoxPanel;
    }


    // ===========================================================================
    // INTERFACE IMPLEMENTATIONS


    public void addListeners() {
    }


    public void initialiseGUI() throws Exception {
        this.setPreferredSize(new Dimension(200, 100));
        this.setMinimumSize(new Dimension(200, 50));

        browsePanel_directory = createBrowsePanel(Language.OPTIONPANEL_OUTPUT, Language.OPTIONPANEL_OUTPUT_TIP, "Output Directory", new byte[] {FileBrowser.FILTER_ACCEPT_DIRECTORIES});
        browsePanel_tagset = createBrowsePanel(Language.OPTIONPANEL_TAGSET, Language.OPTIONPANEL_TAGSET_TIP, "Tag Set", new byte[] {FileBrowser.FILTER_ACCEPT_DATAFILES, FileBrowser.FILTER_ACCEPT_ALL_FILES});

        evaluationCheckBoxPanel = createCheckBoxPanel(Language.OPTIONPANEL_EVALUATIONTYPES, Language.OPTIONPANEL_EVALUATIONTYPES_TIP, "Evaluation Type");
        logCheckBoxPanel = createCheckBoxPanel(Language.OPTIONPANEL_LOGTYPES, Language.OPTIONPANEL_LOGTYPES_TIP, "Log Type");

        jCheckBox_parsed = new JCheckBox(Language.OPTIONPANEL_PARSED);
        jCheckBox_parsed.setToolTipText(Language.OPTIONPANEL_PARSED_TIP);
        jCheckBox_preprocess = new JCheckBox(Language.OPTIONPANEL_PREPROCESS);
        jCheckBox_preprocess.setToolTipText(Language.OPTIONPANEL_PREPROCESS_TIP);

        this.setLayout(new GridBagLayout());
        this.add(browsePanel_directory, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        this.add(browsePanel_tagset, new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        this.add(evaluationCheckBoxPanel, new GridBagConstraints(0, 2, 1, 1, 1.0, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        this.add(logCheckBoxPanel, new GridBagConstraints(0, 3, 1, 1, 1.0, 0.0, GridBagConstraints.NORTHWEST,
                GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        this.add(jCheckBox_preprocess, new GridBagConstraints(0, 4, 1, 1, 1.0, 0.0, GridBagConstraints.NORTHWEST,
                GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        this.add(jCheckBox_parsed, new GridBagConstraints(0, 5, 1, 1, 1.0, 1.0, GridBagConstraints.NORTHWEST,
                GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }


    public String getID() {
        return ID;
    }


    public void initialiseHistory() {
        jCheckBox_parsed.setSelected(History.getOptionPanelParsed(ID, false));
        jCheckBox_preprocess.setSelected(History.getOptionPanelPreprocess(ID, false));
    }


    public void initialiseLanguage() {
    }
}
