/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.gui;

import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.tree.*;
import javax.swing.*;
import java.io.*;
import java.awt.*;
import javax.swing.event.*;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.*;

/**
 * <b>Description:</b> Used for the GuiTAR tool's GUI.<br>
 * <b>Copyright:</b> Copyright (c) 2005<br>
 * <b>Company:</b> Essex University<br>
 * @author Philippe J. Goux
 * @version 1.0
 */
public class ResultPanel extends JPanel implements Gui {
    // ===========================================================================
    // CONSTANTS
    private static final String ID = "Result Panel";


    // ==========================================================================
    // VARIABLES
    protected DefaultMutableTreeNode rootNode;
    protected DefaultTreeModel treeModel;
    protected JTree tree;
    protected JEditorPane jEditorPane = new JEditorPane();


    // ==========================================================================
    // CONSTRUCTORS
    public ResultPanel() {
        try {
            initialiseGUI();
            initialiseHistory();
            initialiseLanguage();
            addListeners();
        } catch (Exception ex) {
            Local.err.printException(ex);
        }
    }


    // ==========================================================================
    // PUBLIC FUNCTIONS
    public void clearFiles() {
        int childCount = rootNode.getChildCount();
        if (childCount > 0) {
            for (int i = 0; i <= rootNode.getChildCount(); i++) {
                MutableTreeNode node = rootNode.getFirstLeaf();
                treeModel.removeNodeFromParent(node);
            }
        }
    }


    public DefaultMutableTreeNode addFile(File file) {
        return addFile(new FileObject(file));
    }


    public DefaultMutableTreeNode addFile(FileObject file) {
        return addObject(rootNode, file, true);
    }


    public DefaultMutableTreeNode addObject(DefaultMutableTreeNode parent, Object child, boolean shouldBeVisible) {
        DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child);

        if (parent == null) {
            parent = rootNode;
        }

        treeModel.insertNodeInto(childNode, parent, parent.getChildCount());

        //Make sure the user can see the new node.
        if (shouldBeVisible)
            tree.scrollPathToVisible(new TreePath(childNode.getPath()));

        return childNode;
    }


    // ==========================================================================
    // PRIVATE FUNCTIONS
    private JTree createJTree() {
        rootNode = new DefaultMutableTreeNode(new DefaultMutableTreeNode(Language.RESULTPANEL_OUTPUTFILES));
        treeModel = new DefaultTreeModel(rootNode);

        JTree jTree = new JTree(treeModel);
        jTree.setEditable(false);
        jTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        jTree.setShowsRootHandles(true);
        return jTree;
    }


    // ===========================================================================
    // INTERFACE IMPLEMENTATIONS
    public void addListeners() {
        tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
                if (node == null)
                    return;

                try {
                    FileObject file = (FileObject) node.getUserObject();
                    jEditorPane.setPage(file.getHtml().toURL());
                } catch (IOException ex) {
                    Local.err.printException(ex);
                    jEditorPane.setText(Language.ERROR_COULDNOTDISPLAYRESULTS);
                } catch (ClassCastException ex) { /* Ignore error */}
            }
        });

    }


    public void initialiseGUI() throws Exception {
        tree = createJTree();

        this.setLayout(new BorderLayout());
        JSplitPane jSplitPane = new JSplitPane();

        this.add(jSplitPane, java.awt.BorderLayout.CENTER);
        jSplitPane.add(Utils.putInJScrollPane(tree), JSplitPane.LEFT);
        jSplitPane.add(Utils.putInJScrollPane(jEditorPane), JSplitPane.RIGHT);
        jSplitPane.setDividerLocation(200);
    }


    public String getID() {
        return "";
    }


    public void initialiseHistory() {
    }


    public void initialiseLanguage() {
    }
}
