/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.gui;

import java.awt.*;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.util.Local;

/**
 * <b>Description:</b> Used for the GuiTAR tool's GUI.<br>
 * <b>Copyright:</b> Copyright (c) 2005<br>
 * <b>Company:</b> Essex University<br>
 * @author Philippe J. Goux
 * @version 1.0
 */

public abstract class Utils {
    public static ImageIcon getImageIcon(String imageName) {
        ImageIcon image = new ImageIcon();
        try {
            image = new ImageIcon(uk.ac.essex.malexa.nlp.dp.GuiTAR.gui.MainFrame.class.getResource("images/" + imageName));
        } catch (NullPointerException e) {
            /* Simply ignore error. */
            Local.err.printException(e);
        }
        return image;
    }


    public static void showDialogBox(JDialog dlg) {
        centerWindow(dlg);
        dlg.setModal(true);
        dlg.pack();
        dlg.setVisible(true);
    }


    public static void centerWindow(Component win) {
        Dimension winSize = win.getPreferredSize();
        Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize();
        if (winSize.height > scrSize.height)
            winSize.height = scrSize.height;
        if (winSize.width > scrSize.width)
            winSize.width = scrSize.width;

        win.setLocation((scrSize.width - winSize.width) / 2,
                        (scrSize.height - winSize.height) / 2);
    }


    public static JComponent createBorder(JComponent jPanel, String title) {
        TitledBorder modificationTitle = new TitledBorder(BorderFactory.
                createLineBorder(SystemColor.inactiveCaption, 2),
                "  " + title + "  ");
        jPanel.setBorder(BorderFactory.createCompoundBorder(modificationTitle,
                BorderFactory.createEmptyBorder(0, 5, 5, 5)));
        return jPanel;
    }


    public static JScrollPane putInJScrollPane(Component comp) {
        JScrollPane jScrollPane = new JScrollPane();
        jScrollPane.setBorder(BorderFactory.createLoweredBevelBorder());
        jScrollPane.getViewport().add(comp, null);
        return jScrollPane;
    }


    public static DefaultListModel listToListModel(List list) {
        DefaultListModel listModel = new DefaultListModel();
        if (list != null) {
            for (Iterator i = list.iterator(); i.hasNext(); )
                listModel.addElement(i.next());
        }
        return listModel;
    }


    public static DefaultListModel listToListModel(List list,
            DefaultListModel listModel) {
        if (list != null) {
            for (Iterator i = list.iterator(); i.hasNext(); )
                listModel.addElement(i.next());
        }
        return listModel;
    }


    public static List listModelToList(DefaultListModel listModel) {
        List list = new ArrayList();
        if (listModel != null) {
            int size = listModel.size();
            for (int i = 0; i < size; i++)
                list.add(listModel.getElementAt(i));
        }
        return list;
    }
}
