/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.la;

/**
 * A class that provides the implementation of the methods defined by intefaces Sense and LexicalEntry.
 */
public class ImplementerLexicalWord implements Sense, LexicalEntry {

	protected String wordString;
	protected String lemma;
	protected String partOfSpeech;
	protected Object sense;

		public ImplementerLexicalWord( String word, String lemma, String partOfSpeech, Object sense ) {
				this.wordString = word;
				this.lemma = lemma;
				this.partOfSpeech = partOfSpeech;
				this.sense = sense;
		}

	/////////////METHODS FROM INTERFACE SENSE//////////////

	/**
	 * Returns the sense of the word.
	 * @return Object The sense
	 */
	public Object getSense() {
		return sense;
	}

	/**
	 * Returns the similarity between Sense s and this sense.
	 * Depending on the implementation might range from [0 - 1] or [-1 - 1].
	 * @param s A sense to be compared with this sense
	 * @return double The similarity between s and this sense
	 */
	public double senseSimilarity( Sense s ) {
		/** IMPELEMENTATION NEEDED **/ return 0.0;
	}

	/**
	 * Tests for equality between Object o and this object.
	 * @param o The object to be compared with this object
	 * @return boolean True if the test succeeds, false otherwise
	 */
	public boolean equals( Object o ) {

		return (o instanceof ImplementerLexicalWord) && sense.equals(((ImplementerLexicalWord)o).sense );
	}

	/**
	 * It seems that this method has to be overriden as well, in order for JAVA's Collections Framework to work properly.
	 * @return int The hash code of a given sense
	 */
	public int hashCode() {
		return sense.hashCode();
	}

	/////////////METHODS FROM INTERFACE LEXICALENTRY///////////////

	/**
	 * Returns the original word.
	 * @return String The word entry
	 */
	public String getWordString() {
		return wordString;
	}

	/**
	 * Returns the part-of-speech of the word.
	 * @return String The pos
	 */
	public String getPartOfSpeech() {
		return partOfSpeech;
	}

	/**
	 * Returns the dictionary entry of the word (base form) after disambiguation.
	 * @return String The lemma
	 */
	public String getLemma() {
		return lemma;
	}


} //End class ImplementerLexicalWord

