/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.util;

import java.io.File;

/**
 * <b>Description:</b> This class should contain all the history
 * used by GuiTAR.<br>
 * <b>Copyright:</b> Copyright (c) 2005<br>
 * <b>Company:</b> Essex University<br>
 * @author Philippe J. Goux
 * @version 1.0
 */
public abstract class History {
    /**
     * The IniFile class has been designed to be able to retrieve
     * or set a specific value from a given INI file. These files are text based
     * and can be easily modified by a user. This may be useful for cases in
     * which the user may be required to change that file but it may also cause
     * trouble if that file is corrupted by the user.<br>
     * This specific <code>IniFile</code> object represents the INI file used
     * to store Tag Sets and Tag Groups.
     *
     * @see uk.ac.essex.malexa.nlp.dp.GuiTAR.util.IniFile#getKeyValue(String, String, String)
     * @see uk.ac.essex.malexa.nlp.dp.GuiTAR.util.IniFile#getKeyValues(String, String, String[])
     * @see uk.ac.essex.malexa.nlp.dp.GuiTAR.util.IniFile#setKeyValue(String, String, String)
     * @see uk.ac.essex.malexa.nlp.dp.GuiTAR.util.IniFile#setKeyValues(String, String, String[])
     */
    private static final IniFile systemFile = new IniFile(new File(Local.CURRENT_DIR + Local.FILE_SEPARATOR + "system.dat"), false);
    public static void setCreateFile(boolean b) {
        systemFile.setCreateFile(b);
    }


    public static String getLanguage() {
        return systemFile.getKeyValue("System", "LANGUAGE", "English (uk)");
    }


    public static void setLanguage(String language) {
        systemFile.setKeyValue("System", "LANGUAGE", language);
    }


    public static void storeError(String[] stackTrace) {
        systemFile.setKeyValues("Error Traces", "ERROR_" + systemFile.getKeys("Error Traces").length, stackTrace);
    }


    public static String[] getBrowsePanelFiles(String panelID, String[] files) {
        return systemFile.getKeyValues("BrowsePanel::" + panelID, "FILES", files);
    }


    public static void setBrowsePanelFiles(String panelID, String[] files) {
        systemFile.setKeyValues("BrowsePanel::" + panelID, "FILES", files);
    }


    public static boolean getCheckboxPanelSelected(String panelID, boolean selected) {
        return Boolean.valueOf(systemFile.getKeyValue("CheckboxPanel::" + panelID, "SELECTED", String.valueOf(selected))).booleanValue();
    }


    public static void setCheckboxPanelSelected(String panelID, boolean selected) {
        systemFile.setKeyValue("CheckboxPanel::" + panelID, "SELECTED", String.valueOf(selected));
    }


    public static String getCheckboxPanelValue(String panelID, String value) {
        return systemFile.getKeyValue("CheckboxPanel::" + panelID, "VALUE", value);
    }


    public static void setCheckboxPanelValue(String panelID, String value) {
        systemFile.setKeyValue("CheckboxPanel::" + panelID, "VALUE", value);
    }


    public static boolean getOptionPanelPreprocess(String panelID, boolean preprocess) {
        return Boolean.valueOf(systemFile.getKeyValue("OptionPanel::" + panelID, "PREPROCESS", String.valueOf(preprocess))).booleanValue();
    }


    public static void setOptionPanelPreprocess(String panelID, boolean preprocess) {
        systemFile.setKeyValue("OptionPanel::" + panelID, "PREPROCESS", String.valueOf(preprocess));
    }


    public static boolean getOptionPanelParsed(String panelID, boolean parsed) {
        return Boolean.valueOf(systemFile.getKeyValue("OptionPanel::" + panelID, "PARSED", String.valueOf(parsed))).booleanValue();
    }


    public static void setOptionPanelParsed(String panelID, boolean parsed) {
        systemFile.setKeyValue("OptionPanel::" + panelID, "PARSED", String.valueOf(parsed));
    }
}
