/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.util;

import java.io.File;

/**
 * <b>Description:</b> This class should contain all the sentences
 * used by GuiTAR.<br>
 * <b>Copyright:</b> Copyright (c) 2005<br>
 * <b>Company:</b> Essex University<br>
 * @author Philippe J. Goux
 * @version 1.0
 */
public abstract class Language {
    private static String language = History.getLanguage();

    public static String BROWSEPANEL_XMLFILES;
    public static String BROWSEPANEL_XMLFILES_TIP;
    public static String BUTTON_BROWSE;
    public static String BUTTON_CLOSE;
    public static String BUTTON_CLOSE_TIP;
    public static String BUTTON_OK;
    public static String BUTTON_PROCESS;
    public static String BUTTON_PROCESS_TIP;
    public static String DESCRIPTION_ALL_FILES;
    public static String DESCRIPTION_DATA_FILES;
    public static String DESCRIPTION_DIRECTORIES_ONLY;
    public static String DESCRIPTION_IMAGES_ONLY;
    public static String DESCRIPTION_XML_FILES;
    public static String ERROR_PARSEDFILENOTFOUND;
    public static String ERROR_COULDNOTDISPLAYRESULTS;
    public static String FILEMENU_ABOUT;
    public static String FILEMENU_EXIT;
    public static String FILEMENU_FILE;
    public static String FILEMENU_HELP;
    public static String FILEMENU_LANGUAGE;
    public static String FLAG_FILENAME;
    public static String LABEL_OPTION;
    public static String LABEL_RESULT;
    public static String TITLE_ABOUT;
    public static String TITLE_GUITAR;
    public static String TRANSLATOR_NAME;
    public static String OPTIONPANEL_OUTPUT;
    public static String OPTIONPANEL_OUTPUT_TIP;
    public static String OPTIONPANEL_TAGSET;
    public static String OPTIONPANEL_TAGSET_TIP;
    public static String OPTIONPANEL_EVALUATIONTYPES;
    public static String OPTIONPANEL_EVALUATIONTYPES_TIP;
    public static String OPTIONPANEL_LOGTYPES;
    public static String OPTIONPANEL_LOGTYPES_TIP;
    public static String OPTIONPANEL_PARSED;
    public static String OPTIONPANEL_PARSED_TIP;
    public static String OPTIONPANEL_PREPROCESS;
    public static String OPTIONPANEL_PREPROCESS_TIP;
    public static String RESULTPANEL_OUTPUTFILES;
    public static String USAGE_AUTHORS;
    public static String USAGE_TOOL;
    public static String USAGE_VERSION;
    public static String USAGE_COMPANY;
    public static String USAGE_TRANSLATOR;
    public static String USAGE_EXAMPLE;
    public static String USAGE_OPTIONS;
    public static String USAGE_USAGE;
    public static String USAGE_HELP;
    public static String USAGE_PARSED;
    public static String USAGE_HTMLOUTPUT;
    public static String USAGE_EVALUATE;
    public static String USAGE_PREPROCESS;
    public static String USAGE_INPUT;
    public static String USAGE_VERBOSE;
    public static String USAGE_VERSION_DETAIL;
    public static String USAGE_LOG;
    public static String USAGE_TAGSET;
    public static String USAGE_FILE;
    public static String USAGE_MAXENT;
    public static String USAGE_SVM;

    /**
     * The IniFile class has been designed to be able to retrieve
     * or set a specific value from a given INI file. These files are text based
     * and can be easily modified by a user. This may be useful for cases in
     * which the user may be required to change that file but it may also cause
     * trouble if that file is corrupted by the user.<br>
     * This specific <code>IniFile</code> object represents the INI file used
     * to store Tag Sets and Tag Groups.
     *
     * @see uk.ac.essex.malexa.nlp.dp.GuiTAR.util.IniFile#getKeyValue(String, String, String)
     * @see uk.ac.essex.malexa.nlp.dp.GuiTAR.util.IniFile#getKeyValues(String, String, String[])
     * @see uk.ac.essex.malexa.nlp.dp.GuiTAR.util.IniFile#setKeyValue(String, String, String)
     * @see uk.ac.essex.malexa.nlp.dp.GuiTAR.util.IniFile#setKeyValues(String, String, String[])
     */
    private static IniFile languageFile = new IniFile(new File(Local.CURRENT_DIR + Local.FILE_SEPARATOR + "languages.ini"), false) {
        protected void initialise() {
            // Keep these two line in front
            FLAG_FILENAME = this.getKeyValue(language, "FLAG_FILENAME", "flag_Uk.png");
            TRANSLATOR_NAME = this.getKeyValue(language, "TRANSLATOR_NAME", "Philippe J. Goux");

            // Try to have alphabetical order
            BROWSEPANEL_XMLFILES = this.getKeyValue(language, "BROWSEPANEL_XMLFILES", "XML files");
            BROWSEPANEL_XMLFILES_TIP = this.getKeyValue(language, "BROWSEPANEL_XMLFILES_TIP", "Select the MAS-XML files to be processed.");
            BUTTON_BROWSE = this.getKeyValue(language, "BUTTON_BROWSE", "Browse");
            BUTTON_CLOSE = this.getKeyValue(language, "BUTTON_CLOSE", "Close");
            BUTTON_CLOSE_TIP = this.getKeyValue(language, "BUTTON_CLOSE_TIP", "Exit the GuiTAR tool.");
            BUTTON_OK = this.getKeyValue(language, "BUTTON_OK", "Ok");
            BUTTON_PROCESS = this.getKeyValue(language, "BUTTON_PROCESS", "Process");
            BUTTON_PROCESS_TIP = this.getKeyValue(language, "BUTTON_PROCESS_TIP", "Process the selected MAS-XML files.");
            DESCRIPTION_ALL_FILES = this.getKeyValue(language, "DESCRIPTION_ALL_FILES", "all files");
            DESCRIPTION_DATA_FILES = this.getKeyValue(language, "DESCRIPTION_DATA_FILES", "data files");
            DESCRIPTION_DIRECTORIES_ONLY = this.getKeyValue(language, "DESCRIPTION_DIRECTORIES_ONLY", "directories only");
            DESCRIPTION_IMAGES_ONLY = this.getKeyValue(language, "DESCRIPTION_IMAGES_ONLY", "images only");
            DESCRIPTION_XML_FILES = this.getKeyValue(language, "DESCRIPTION_XML_FILES", "XML files");
            ERROR_COULDNOTDISPLAYRESULTS = this.getKeyValue(language, "ERROR_COULDNOTDISPLAYRESULTS", "Error: Could not display results...");
            ERROR_PARSEDFILENOTFOUND = this.getKeyValue(language, "ERROR_PARSEDFILENOTFOUND", "Could not find file for parsed evaluation");
            FILEMENU_ABOUT = this.getKeyValue(language, "FILEMENU_ABOUT", "About");
            FILEMENU_EXIT = this.getKeyValue(language, "FILEMENU_EXIT", "Exit");
            FILEMENU_FILE = this.getKeyValue(language, "FILEMENU_FILE", "File");
            FILEMENU_HELP = this.getKeyValue(language, "FILEMENU_HELP", "Help");
            FILEMENU_LANGUAGE = this.getKeyValue(language, "FILEMENU_LANGUAGE", "Language");
            LABEL_OPTION = this.getKeyValue(language, "LABEL_OPTION", "Options");
            LABEL_RESULT = this.getKeyValue(language, "LABEL_RESULT", "Results");
            OPTIONPANEL_EVALUATIONTYPES = this.getKeyValue(language, "OPTIONPANEL_EVALUATIONTYPES", "Evaluation Types");
            OPTIONPANEL_EVALUATIONTYPES_TIP = this.getKeyValue(language, "OPTIONPANEL_EVALUATIONTYPES_TIP", "Select the tag which should be evaluated.");
            OPTIONPANEL_LOGTYPES = this.getKeyValue(language, "OPTIONPANEL_LOGTYPES", "Log Types");
            OPTIONPANEL_LOGTYPES_TIP = this.getKeyValue(language, "OPTIONPANEL_LOGTYPES_TIP", "Enable verbose on given classes and log information (if no class is selected, verbose enabled for all).");
            OPTIONPANEL_OUTPUT = this.getKeyValue(language, "BROWSEPANEL_OUTPUT", "Output Directory");
            OPTIONPANEL_OUTPUT_TIP = this.getKeyValue(language, "OPTIONPANEL_OUTPUT_TIP", "Select the process' output directory (default = process launch directory).");
            OPTIONPANEL_PARSED = this.getKeyValue(language, "OPTIONPANEL_PARSED", "Use parsed evaluation file");
            OPTIONPANEL_PARSED_TIP = this.getKeyValue(language, "OPTIONPANEL_PARSED_TIP", "Enable parse attribute for evaluation.");
            OPTIONPANEL_PREPROCESS = this.getKeyValue(language, "OPTIONPANEL_PREPROCESS", "Preprocess files");
            OPTIONPANEL_PREPROCESS_TIP = this.getKeyValue(language, "OPTIONPANEL_PREPROCESS_TIP", "Pre-process xml file, adding attributes such as person, number and gender.");
            OPTIONPANEL_TAGSET = this.getKeyValue(language, "BROWSEPANEL_TAGSET", "Tag Set");
            OPTIONPANEL_TAGSET_TIP = this.getKeyValue(language, "OPTIONPANEL_TAGSET_TIP", "Select the file containing the tag set to use (default = XELDA tag set).");
            RESULTPANEL_OUTPUTFILES = this.getKeyValue(language, "RESULTPANEL_OUTPUTFILES", "Output Files");
            TITLE_ABOUT = this.getKeyValue(language, "TITLE_ABOUT", "About");
            TITLE_GUITAR = this.getKeyValue(language, "TITLE_GUITAR", "General Tool for Anaphora Resolution");

            // Try to keep in apparition order
            USAGE_TOOL = this.getKeyValue(language, "USAGE_TOOL", "Tool");
            USAGE_VERSION = this.getKeyValue(language, "USAGE_VERSION", "Version");
            USAGE_COMPANY = this.getKeyValue(language, "USAGE_COMPANY", "Company");
            USAGE_AUTHORS = this.getKeyValue(language, "USAGE_AUTHORS", "Authors");
            USAGE_TRANSLATOR = this.getKeyValue(language, "USAGE_TRANSLATOR", "Translator");
            USAGE_USAGE = this.getKeyValue(language, "USAGE_USAGE", "Usage");
            USAGE_EXAMPLE = this.getKeyValue(language, "USAGE_EXAMPLE", "e.g.");
            USAGE_OPTIONS = this.getKeyValue(language, "USAGE_OPTIONS", "where options include");
            USAGE_HELP = this.getKeyValue(language, "USAGE_HELP", "print this help message.");
            USAGE_VERSION_DETAIL = this.getKeyValue(language, "USAGE_VERSION_DETAIL", "print product version and exit.");
            USAGE_PARSED = this.getKeyValue(language, "USAGE_PARSED", "enable parse attribute for evaluation.");
            USAGE_HTMLOUTPUT = this.getKeyValue(language, "USAGE_HTMLOUTPUT", "create \"html\" file containing output results.");
            USAGE_EVALUATE = this.getKeyValue(language, "USAGE_EVALUATE", "run evaluation. This must be followed by tag type.");
            USAGE_PREPROCESS = this.getKeyValue(language, "USAGE_PREPROCESS", "pre-process xml file, adding attributes such as person, number and gender.");
            USAGE_VERBOSE = this.getKeyValue(language, "USAGE_VERBOSE", "enable verbose output for selected class. If no class is selected, verbose will be enabled for all.");
            USAGE_INPUT = this.getKeyValue(language, "USAGE_INPUT", "select the MAS-XML files to be processed.");
            USAGE_LOG = this.getKeyValue(language, "USAGE_LOG", "log verbose information to files.");
            USAGE_TAGSET = this.getKeyValue(language, "USAGE_TAGSET", "select file containing the tag set to use. If none is given, default file will be generated.");
            USAGE_FILE = this.getKeyValue(language, "USAGE_FILE", "the name of a file that contains the list of names of the MAS-XML files to be processed.");
            USAGE_SVM = this.getKeyValue(language, "USAGE_SVM", "select the model file name for an SVM classifier trained with LIBSVM software (the extensions .model and .range are assumed and should be omitted when giving the model file name).");
            USAGE_MAXENT = this.getKeyValue(language, "USAGE_MAXENT", "select the model file name for a MaxEnt classifier trained with openNLP's MaxEnt package (the extension .model is assumed and should be omitted when giving the model file name).");
        }
    };

    public static void setCreateFile(boolean b) {
        languageFile.setCreateFile(b);
    }


    public static void setLanguageFile(File file) {
        languageFile.setIniFile(file);
    }


    public static String[] getLanguages() {
        return languageFile.getCathegories();
    }


    public static String getValueInLanguage(String language, String valueKey, String defaultValue) {
        return languageFile.getKeyValue(language, valueKey, defaultValue);
    }


    public static String getLanguage() {
        return language;
    }


    public static void setLanguage(String languageName) {
        language = languageName;
        History.setLanguage(languageName);
        languageFile.initialise();
    }


    /**
     * Returns the a string explaining how to use the GuiTAR tool.
     * @return String
     */
    public static String getUsage() {
        return USAGE_USAGE + ":" + Local.NEWLINE +
                Local.NEWLINE +
                "java -jar GuiTAR.jar [-options] -input file1 [file2 [...]]" + Local.NEWLINE +
                Local.NEWLINE +
                USAGE_OPTIONS + ":" + Local.NEWLINE +
                "   -help or -h  \t: " + USAGE_HELP + Local.NEWLINE +
                "   -version     \t: " + USAGE_VERSION_DETAIL + Local.NEWLINE +
                "   -parsed      \t: " + USAGE_PARSED + Local.NEWLINE +
                "   -html        \t: " + USAGE_HTMLOUTPUT + Local.NEWLINE +
                "   -evaluate    \t: " + USAGE_EVALUATE + Local.NEWLINE +
                "                \t  " + USAGE_EXAMPLE + ": -evaluate pn the-pn" + Local.NEWLINE +
                "   -preprocess  \t: " + USAGE_PREPROCESS + Local.NEWLINE +
                "   -verbose     \t: " + USAGE_VERBOSE + Local.NEWLINE +
                "                \t  " + USAGE_EXAMPLE + ": -verbose PreProSyntacticHeuristics" + Local.NEWLINE +
                "   -log         \t: " + USAGE_LOG + Local.NEWLINE +
                "                \t  " + USAGE_EXAMPLE + ": -verbose DefiniteDescription ProperName -log" + Local.NEWLINE +
                "   -tagset or -t\t: " + USAGE_TAGSET + Local.NEWLINE +
                "   -input or -i \t: " + USAGE_INPUT + Local.NEWLINE +
                "   -file or -f  \t: " + USAGE_FILE + Local.NEWLINE +
                "   -maxent      \t: " + USAGE_MAXENT + Local.NEWLINE +
                "   -svm         \t: " + USAGE_SVM + Local.NEWLINE +
                Local.NEWLINE;
    }


    /**
     * Returns the a string describing the GuiTAR tool.
     * @return String
     */
    public static String getDescription() {
        return
                USAGE_TOOL + ": GuiTAR (" + TITLE_GUITAR + ")" + Local.NEWLINE +
                USAGE_VERSION + ": " + Local.VERSION + Local.NEWLINE +
                USAGE_COMPANY + ": Essex University" + Local.NEWLINE +
                "Copyright (c) 2003, 2004, 2005, 2006" + Local.NEWLINE +
                USAGE_AUTHORS + ": " + "Massimo Poesio; Mijail Alexandrov-Kabadjov; Philippe J. Goux;" + Local.NEWLINE;
    }


    /**
     * Returns the translator of the current language.
     * @return String
     */
    public static String getTranslator() {
        return USAGE_TRANSLATOR + ": " + TRANSLATOR_NAME + Local.NEWLINE;
    }
}
