/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.util;

import java.io.File;
import uk.ac.essex.malexa.nlp.dp.GuiTAR.*;
import java.util.Arrays;

/**
 * <b>Description:</b> This class was created to contain any global variables,
 * objects or method that may required by many other classes within
 * the GuiTAR tool.<br>
 * <b>Copyright:</b> Copyright (c) 2005<br>
 * <b>Company:</b> Essex University<br>
 * @author Philippe J. Goux
 * @version 1.0
 */
public abstract class Local {
    // ===========================================================================
    // CONSTANTS
    /**
     * Returns the file that should contain the Anaphora Resolution Evaluation log.
     */
    public static final String VERSION = "3.0.3";

    /**
     * Returns the path to directory the application was started in,
     * defined by the system property <code>user.dir</code>.
     */
    public static final String CURRENT_DIR = System.getProperty("user.dir");
    /**
     * Returns the file separator used in the current system,
     * defined by the system property <code>file.separator</code>.
     */
    public static final String FILE_SEPARATOR = System.getProperty("file.separator");
    /**
     * Returns the line separator used in the current system,
     * defined by the system property <code>line.separator</code>.
     */
    public static final String NEWLINE = System.getProperty("line.separator");

    /**
     * Returns the file that should contain the PreProSyntacticHeuristics log.
     */
    public static final File FILE_LOG_PREPROCESS = new File("PreProSyntacticHeuristics.log");

    /**
     * Returns the file that should contain the Anaphora Resolution log.
     */
    public static final File FILE_LOG_PROCESS = new File("AnaphoraResolution.log");

    /**
     * Returns the file that should contain the Anaphora Resolution Evaluation log.
     */
    public static final File FILE_LOG_EVALUATION = new File("AnaphoraResolutionEvaluation.csv");


    // ===========================================================================
    // VARIABLES
    /**
     * Enables Enables the GuiTAR user to select which type of traces
     * to have displayed<br>
     * See the <code>println</code> methods in class <code>Log</code>.
     *
     * @see     uk.ac.essex.malexa.nlp.dp.GuiTAR.util.Log#println()
     * @see     uk.ac.essex.malexa.nlp.dp.GuiTAR.util.Log#println(Object)
     * @see     uk.ac.essex.malexa.nlp.dp.GuiTAR.util.Log#println(String, Object)
     * @see     uk.ac.essex.malexa.nlp.dp.GuiTAR.util.Log#print(Object)
     * @see     uk.ac.essex.malexa.nlp.dp.GuiTAR.util.Log#print(String, Object)
     */
    public static Log out = new Log(System.out);
    public static Log err = new Log(System.err);


    // ===========================================================================
    // PUBLIC FUNCTIONS
    /**
     * Returns wether the String str is contained within the String[] array.
     * The search is not case sensitive.
     * @param array String[]
     * @param str String
     * @return boolean
     */
    public static boolean containsIgnoreCase(String[] array, String str) {
        for (int i = 0; i < array.length; i++) {
            if (array[i].equalsIgnoreCase(str))
                return true;
        }
        return false;
    }


    /**
     * Returns wether the String str is contained within the String[] array.
     * The search is case sensitive.
     * @param array String[]
     * @param str String
     * @return boolean
     */
    public static boolean contains(String[] array, String str) {
        for (int i = 0; i < array.length; i++) {
            if (array[i].equals(str))
                return true;
        }
        return false;
    }
}
