/**************************************************************************
*  GuiTAR - A General Tool for Anaphora Resolution
*  Copyright (C) 2004-2007
*  Mijail Kabadjov, Massimo Poesio, Olivia Sanchez-Graillet, Philippe Goux
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**************************************************************************/

package uk.ac.essex.malexa.nlp.dp.GuiTAR.util;

import java.io.File;

/**
 * <b>Description:</b> This class should contain all the required tags
 * used by GuiTAR which may be modified by the user.<br>
 * <b>Copyright:</b> Copyright (c) 2005<br>
 * <b>Company:</b> Essex University<br>
 * @author Philippe J. Goux
 * @version 1.0
 */
public abstract class TagSet {
    //Google
    public static String GOOGLE_KEY;

    //Tagset
    public static String POS_PROPERTY_NAME;
    public static String TAGSET_NAME;
    public static String CARDINAL[];
    public static String CON_COORDINATING[];
    public static String COMMA[];
    public static String PUNCTUATION[];
    public static String WH_PRONOUN[];
    public static String DETERMINER[];
    public static String DETERMINER2[];
    public static String POSSESSIVE_PRONOUN[];
    public static String POSSESSIVE[];
    public static String PREPOSITION[];
    public static String ADJECTIVE_GENERAL[];
    public static String ADJECTIVE_COMPARATIVE[];
    public static String ADJECTIVE_SUPERLATIVE[];
    public static String COMMON_NOUN_SINGULAR[];
    public static String COMMON_NOUN_PLURAL[];
    public static String PROPER_NOUN_SINGULAR[];
    public static String PROPER_NOUN_PLURAL[];
    public static String PERSONAL_PRONOUN[];
    public static String PERSONAL_PRONOUN_PLURAL[];
    public static String PERSONAL_PRONOUN_OBJECT[];
    public static String REFLEXIVE_PRONOUN[];
    public static String REFLEXIVE_PRONOUN_PLURAL[];
    public static String ADVERB_GENERAL[];
    public static String ADVERB_COMPARATIVE[];
    public static String ADVERB_SUPERLATIVE[];
    public static String TOBE_PAST_TENSE_SINGULAR[];
    public static String TOBE_PRESENT_1_SINGULAR[];
    public static String VERB_PRESENT_3_SINGULAR[];
    public static String LIST_VERB_TAGS[];

    //The Tag NR was used by XELDA to tag East and West in East/West Coast
    public static String VALID_ATTR_TAGS[];

    public static String VALID_PROPER_NOUN_TAGS[];

    /**
     * The IniFile class has been designed to be able to retrieve
     * or set a specific value from a given INI file. These files are text based
     * and can be easily modified by a user. This may be useful for cases in
     * which the user may be required to change that file but it may also cause
     * trouble if that file is corrupted by the user.<br>
     * This specific <code>IniFile</code> object represents the INI file used
     * to store Tag Sets and Tag Groups.
     *
     * @see uk.ac.essex.malexa.nlp.dp.GuiTAR.util.IniFile#getKeyValue(String, String, String)
     * @see uk.ac.essex.malexa.nlp.dp.GuiTAR.util.IniFile#getKeyValues(String, String, String[])
     * @see uk.ac.essex.malexa.nlp.dp.GuiTAR.util.IniFile#setKeyValue(String, String, String)
     * @see uk.ac.essex.malexa.nlp.dp.GuiTAR.util.IniFile#setKeyValues(String, String, String[])
     */
    private static IniFile tagSetFile = new IniFile(new File(Local.CURRENT_DIR + Local.FILE_SEPARATOR + "tagSet.ini"), false) {
        protected void initialise() {
            GOOGLE_KEY = this.getKeyValue("Google API", "GOOGLE_KEY", "");

            POS_PROPERTY_NAME = this.getKeyValue("Xml Tags", "POS_PROPERTY_NAME", "Xpos");

            TAGSET_NAME = this.getKeyValue("Tagset", "TAGSET_NAME", "XELDA"); //name of the tagset used
            CARDINAL = this.getKeyValues("Tagset", "CARDINAL", new String[] {"CD"}); //(number, cardinal (four))
            CON_COORDINATING = this.getKeyValues("Tagset", "CON_COORDINATING", new String[] {"CC"}); //(conjunction, coordinating (and))
            COMMA = this.getKeyValues("Tagset", "COMMA", new String[] {"CM"});
            PUNCTUATION = this.getKeyValues("Tagset", "PUNCTUATION", new String[] {"PUNCT"});
            WH_PRONOUN = this.getKeyValues("Tagset", "WH_PRONOUN", new String[] {"WPS"}); //( who )
            DETERMINER = this.getKeyValues("Tagset", "DETERMINER", new String[] {"AT"}); //(a, the ) - For XELDA these are articles
            DETERMINER2 = this.getKeyValues("Tagset", "DETERMINER2", new String[] {"DT"}); //(this, that) - For XELDA these are determiners
            POSSESSIVE_PRONOUN = this.getKeyValues("Tagset", "POSSESSIVE_PRONOUN", new String[] {"PP$"}); //(my, his)
            POSSESSIVE = this.getKeyValues("Tagset", "POSSESSIVE", new String[] {"POS"}); //apostrophe
            PREPOSITION = this.getKeyValues("Tagset", "PREPOSITION", new String[] {"IN"}); //(on, of)
            ADJECTIVE_GENERAL = this.getKeyValues("Tagset", "ADJECTIVE_GENERAL", new String[] {"JJ"}); //(near)
            ADJECTIVE_COMPARATIVE = this.getKeyValues("Tagset", "ADJECTIVE_COMPARATIVE", new String[] {"JJR"}); //(nearer)
            ADJECTIVE_SUPERLATIVE = this.getKeyValues("Tagset", "ADJECTIVE_SUPERLATIVE", new String[] {"JJT"}); //(nearest)
            COMMON_NOUN_SINGULAR = this.getKeyValues("Tagset", "COMMON_NOUN_SINGULAR", new String[] {"NN"});
            COMMON_NOUN_PLURAL = this.getKeyValues("Tagset", "COMMON_NOUN_PLURAL", new String[] {"NNS"});
            PROPER_NOUN_SINGULAR = this.getKeyValues("Tagset", "PROPER_NOUN_SINGULAR", new String[] {"NP"});
            PROPER_NOUN_PLURAL = this.getKeyValues("Tagset", "PROPER_NOUN_PLURAL", new String[] {"NPS"});
            PERSONAL_PRONOUN = this.getKeyValues("Tagset", "PERSONAL_PRONOUN", new String[] {"PPS"}); //(I, he)
            PERSONAL_PRONOUN_PLURAL = this.getKeyValues("Tagset", "PERSONAL_PRONOUN_PLURAL", new String[] {"PPSS"}); //(we, they)
            PERSONAL_PRONOUN_OBJECT = this.getKeyValues("Tagset", "PERSONAL_PRONOUN_OBJECT", new String[] {"PPO"}); //in object position
            REFLEXIVE_PRONOUN = this.getKeyValues("Tagset", "REFLEXIVE_PRONOUN", new String[] {"PPL"}); //(myself, himself)
            REFLEXIVE_PRONOUN_PLURAL = this.getKeyValues("Tagset", "REFLEXIVE_PRONOUN_PLURAL", new String[] {"PPLS"}); //(ourselves)
            ADVERB_GENERAL = this.getKeyValues("Tagset", "ADVERB_GENERAL", new String[] {"RB"}); //(chronically, deep)
            ADVERB_COMPARATIVE = this.getKeyValues("Tagset", "ADVERB_COMPARATIVE", new String[] {"RBR"}); //(easier, sooner)
            ADVERB_SUPERLATIVE = this.getKeyValues("Tagset", "ADVERB_SUPERLATIVE", new String[] {"RBS"}); //(easiest, soonest)
            TOBE_PAST_TENSE_SINGULAR = this.getKeyValues("Tagset", "TOBE_PAST_TENSE_SINGULAR", new String[] {"BEDZ"});
            TOBE_PRESENT_1_SINGULAR = this.getKeyValues("Tagset", "TOBE_PRESENT_1_SINGULAR", new String[] {"BEM"});

            VERB_PRESENT_3_SINGULAR = this.getKeyValues("Tag Groups", "VERB_PRESENT_3_SINGULAR",
                    new String[] {"BEZ", "DOZ", "HVZ", "VBZ"});
            LIST_VERB_TAGS = this.getKeyValues("Tag Groups", "LIST_VERB_TAGS",
                    new String[] {"BE", "BED", "BEDZ", "BEG", "BEM", "BEN", "BER", "BEZ", "DO", "DOD",
                    "DOG", "DOZ", "HV", "HVD", "HVN", "HVZ", "VB", "VBD", "VBG", "VBN", "VBP", "VBZ"});
            //The Tag NR was used by XELDA to tag East and West in East/West Coast
            VALID_ATTR_TAGS = this.getKeyValues("Tag Groups", "VALID_ATTR_TAGS",
                    new String[] {"FW", "JJ", "JJR", "JJT", "NN", "NNS", "NP", "NPS",
                    "NR", "OD", "RB", "RBR", "RBS", "VBD", "VBG", "VBN", "VBP"});
            VALID_PROPER_NOUN_TAGS = this.getKeyValues("Tag Groups", "VALID_PROPER_NOUN_TAGS",
                    new String[] {"NP", "NPS"});
        }
    };

    /**
     * Set tag set to be used.
     * @param file File
     */
    public static void setTagSet(File file) {
        tagSetFile.setIniFile(file);
    }


    /**
     * Set the fact that the file should be created if not found.
     * @param b boolean
     */
    public static void setCreateFile(boolean b) {
        tagSetFile.setCreateFile(b);
    }

}
